package com.hand.hap.activiti.components;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.hand.hap.activiti.approval.dto.ApproveCandidateRule;
import com.hand.hap.activiti.approval.dto.ApproveStrategy;
import com.hand.hap.activiti.approval.dto.BusinessRuleHeader;
import com.hand.hap.activiti.approval.dto.BusinessRuleLine;
import com.hand.hap.activiti.approval.mapper.ApproveCandidateRuleMapper;
import com.hand.hap.activiti.approval.mapper.ApproveStrategyMapper;
import com.hand.hap.activiti.approval.mapper.BusinessRuleHeaderMapper;
import com.hand.hap.activiti.approval.mapper.BusinessRuleLineMapper;
import com.hand.hap.activiti.core.IActivitiConstants;
import com.hand.hap.activiti.custom.IActivitiBean;
import com.hand.hap.activiti.service.IActivitiEntityService;
import com.hand.hap.activiti.util.ActivitiUtils;
import com.hand.hap.core.IRequest;
import com.hand.hap.core.impl.RequestHelper;
import com.hand.hap.hr.dto.Employee;
import com.hand.hap.hr.dto.Position;
import com.hand.hap.hr.mapper.EmployeeMapper;
import com.hand.hap.hr.mapper.PositionMapper;
import hls.core.fnd.dto.HlsEmployee;
import hls.core.fnd.mapper.HlsEmployeeMapper;
import org.activiti.bpmn.model.FormProperty;
import org.activiti.bpmn.model.FormValue;
import org.activiti.bpmn.model.UserTask;
import org.activiti.editor.constants.EditorJsonConstants;
import org.activiti.engine.ActivitiException;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.impl.cfg.ProcessEngineConfigurationImpl;
import org.activiti.engine.impl.el.ExpressionManager;
import org.activiti.engine.impl.identity.Authentication;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.Date;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

/**
 * Created by Qixiangyu on 2017/3/16.
 */
@Component
public class ApprovalRule implements IActivitiBean, IActivitiConstants, EditorJsonConstants {

    public static final String PARENT_POSITION = "PARENT_POSITION";//上级领导审批
    String ID = "id";
    String NAME = "name";
    String CODE = "code";
    String YES = "Y";
    String ENUM_VALUES = "enumValues";
    String ASSIGNEE = "assignee";
    String ASSIGNEE_CODE = "assigneeCode";
    String ASSIGNEE_NAME = "assigneeName";
    String FORM_PROPERTIES = "formProperties";
    String APPROVE_STRATEGY = "approve_strategy";
    String APPROVE_RULE = "approve_rule";
    String DESCRIPTION = "description";
    String RULES = "rules";


    private static final Logger logger = LoggerFactory.getLogger(ApprovalRule.class);

    @Autowired
    private ObjectMapper objectMapper;


    @Autowired
    private ApproveCandidateRuleMapper candidateRuleMapper;

    @Autowired
    private BusinessRuleHeaderMapper businessRuleHeaderMapper;

    @Autowired
    private BusinessRuleLineMapper businessRuleLineMapper;

    @Autowired
    private ApproveStrategyMapper strategyMapper;

    @Autowired
    private PositionMapper positionMapper;

    @Autowired
    private EmployeeMapper employeeMapper;
    @Autowired
    private HlsEmployeeMapper hlsEmployeeMapper;

    @Autowired
    private ProcessEngineConfigurationImpl processEngineConfiguration;

    @Autowired
    private IActivitiEntityService activitiEntityService;


    /**
     * 动态解析表单属性返回多实例的集合
     *
     * @param execution 获取流程信息
     * @return
     */
    public Set<String> getApprovalRule(DelegateExecution execution) {
        // 验证是否是userTask
        Object element = execution.getCurrentFlowElement();
        UserTask task;
        if (element instanceof UserTask) {
            task = (UserTask) element;
        } else {
            logger.error("must be a UserTask can use this expression");
            throw new ActivitiException("must be a UserTask can use this expression");
        }
        return processCandidate(task, execution);
    }

    /**
     * 根据表单属性，动态解析出集合，避免选人重复，返回set集合去重
     *
     * @return set集合，包含解析出来的所有employeeCode
     */
    public Set<String> processCandidate(UserTask task, DelegateExecution execution) {

        ExpressionManager expressionManager = processEngineConfiguration.getExpressionManager();
        Set<String> candidates = new LinkedHashSet<>();
        // 获取候选人规则对应的表单属性
        FormProperty property = ActivitiUtils.getFormPropertyById(task.getFormProperties(), APPROVAL_CANDIDATE);
        List<FormValue> values = property.getFormValues();
        if (values == null) {
            logger.warn("APPROVAL_CANDIDATE has no formValue check formProperty");
            return candidates;
        }
        String strategyCode, jsonVule;
        // 选人策略对象
        ApproveCandidateRule candidateRule;
        // 解析出的候选人
        Object user;
        //Map<String, List> ruleMap;
        JsonNode ruleJson = null;
        for (FormValue value : values) {
            strategyCode = value.getId();
            candidateRule = getCandidateRule(strategyCode);
            //失效或者CODE不正确
            if (candidateRule == null) {
                logger.warn("code :" + strategyCode + " is disable or not exist");
                continue;
            }
            jsonVule = value.getName();
            try {
                ruleJson = objectMapper.readTree(jsonVule);
            } catch (IOException e) {
                logger.warn(e.getMessage());
            }
            if (!hasPermission(ruleJson, execution)) {
                continue;
            }
            ArrayNode parameters = (ArrayNode) ruleJson.get(APPROVAL_PARAMETER);
            // 指定人员
            if (APPROVAL_CANDIDATE_EMP.equalsIgnoreCase(strategyCode)) {
                for (JsonNode para : parameters) {
                    JsonNode codeObj = para.get(APPROVAL_CODE_EMPLOYEE);
                    if (codeObj == null) {
                        logger.warn("employee code is null");
                        continue;
                    }
                    String employeeCode = codeObj.textValue();
                    execution.setVariableLocal(APPROVAL_CODE_EMPLOYEE, employeeCode);
                    Object empCode = expressionManager.createExpression(candidateRule.getExpression())
                            .getValue(execution);
                    candidates.add(empCode.toString());
                }
            } else if (APPROVAL_CANDIDATE_POS.equalsIgnoreCase(strategyCode)) {
                // 指定岗位
                for (JsonNode para : parameters) {
                    JsonNode codeObj = para.get(APPROVAL_CODE_POSITION);
                    if (codeObj == null) {
                        logger.warn("position code is null");
                        continue;
                    }
                    String positionCode = codeObj.textValue();
                    execution.setVariableLocal(APPROVAL_CODE_POSITION, positionCode);
                    List<String> emps = (List<String>) expressionManager
                            .createExpression(candidateRule.getExpression()).getValue(execution);
                    candidates.addAll(emps);
                }
            } else if (PARENT_POSITION.equalsIgnoreCase(strategyCode)) {//上级领导审批(待优化)
                String authenticatedUserId = Authentication.getAuthenticatedUserId();//获取当前提交审批的request请求中线程局部变量所存的用户ID
                String emp = activitiEntityService.getDirector(authenticatedUserId);
                candidates.add(emp);
            } else {
                //如果无法解析表达式，继续找下一个审批规则
                try {
                    user = expressionManager.createExpression(candidateRule.getExpression()).getValue(execution);
                    if (user != null && StringUtils.isNotEmpty(String.valueOf(user))) {
                        if (user instanceof java.util.List) {
                            if (user != null && StringUtils.isNotEmpty(String.valueOf(user))) {
                                for (int i = 0; i < ((List<String>) user).size(); i++) {
                                    candidates.add(((List<String>) user).get(i));
                                }
                            }
                        } else {
                            Set<String> users = org.springframework.util.StringUtils.commaDelimitedListToSet(user.toString());
                            for (String str : users) {
                                candidates.add(str);
                            }
                        }
                    }
                } catch (ActivitiException e) {
                    logger.error(e.getMessage());
                }
            }
        }
        return candidates;
    }

    /**
     * 导入xml流程定义文件时，处理扩展的自定义属性
     *
     * @param content
     *            流程定义文件json对象
     *
     * @return 处理后的json对象
     */
    public ObjectNode processCustomProperties(ObjectNode content) {
        ArrayNode childShaps = (ArrayNode) content.get(EDITOR_CHILD_SHAPES);
        for (JsonNode key : childShaps) {
            ObjectNode child = (ObjectNode) key;
            ObjectNode prop = (ObjectNode) child.get(EDITOR_SHAPE_PROPERTIES);
            ObjectNode formProp = (ObjectNode) prop.get(FORM_PROPERTIES.toLowerCase());
            if (formProp != null && !formProp.isNull()) {
                ArrayNode formPropVaule = (ArrayNode) formProp.get(FORM_PROPERTIES);
                for (JsonNode value : formPropVaule) {
                    String valueId = value.get(ID).asText();
                    switch (valueId) {
                        case APPROVAL_STRATEGY: {
                            String code = value.get(NAME).asText();
                            ApproveStrategy record = new ApproveStrategy();
                            record.setCode(code);
                            record = strategyMapper.selectOne(record);
                            if (record != null) {
                                prop.put(APPROVE_STRATEGY, record.getDescription());
                            }
                            break;
                        }
                        case APPROVAL_WFL_NUM: {
                            JsonNode numObj = value.get(NAME);
                            if (numObj != null && !numObj.isNull()) {
                                Double num = numObj.asDouble();
                                prop.put(APPROVAL_WFL_NUM.toLowerCase(), num);
                            }
                            break;
                        }
                        case APPROVAL_ACTION: {
                            ArrayNode enumValues = (ArrayNode) value.get(ENUM_VALUES);
                            for (JsonNode node : enumValues) {
                                String actionType = node.get(ID).asText();
                                if (ADD_SIGN_FLAG.equalsIgnoreCase(actionType)
                                        || DELEGATE_FLAG.equalsIgnoreCase(actionType)) {
                                    prop.put(actionType.toLowerCase(), YES.equalsIgnoreCase(node.get(NAME).asText()));
                                }
                            }
                            break;
                        }
                        case APPROVAL_CANDIDATE: {
                            prop.set(APPROVE_RULE, processCadidateRuleNode((ObjectNode) value));
                            break;
                        }

                    }

                }
                child.set(EDITOR_SHAPE_PROPERTIES, prop);
            }
        }
        return content;
    }

    private ObjectNode processCadidateRuleNode(ObjectNode value) {
        ArrayNode enumValues = (ArrayNode) value.get(ENUM_VALUES);
        ObjectNode ruleNode = objectMapper.createObjectNode();
        ArrayNode assignmentNode = objectMapper.createArrayNode();
        if (enumValues == null || enumValues.isNull()) {
            return ruleNode;
        }
        for (int i = 0; i < enumValues.size(); i++) {
            ObjectNode node = (ObjectNode) enumValues.get(i);
            String ruleCode = node.get(ID).asText();
            ApproveCandidateRule record = new ApproveCandidateRule();
            record.setCode(ruleCode);

            record = candidateRuleMapper.selectOne(record);

            ObjectNode index = objectMapper.createObjectNode();
            index.put(ID, i);
            index.put(CODE, ruleCode);

            String jsonMessage = node.get(NAME).textValue();
            JsonNode cadidate = null;
            try {
                cadidate = objectMapper.readTree(jsonMessage);
            } catch (IOException e) {
                e.printStackTrace();
            }

            //指定岗位或者指定人
            if (APPOINTED_POSITION.equalsIgnoreCase(ruleCode)) {
                ArrayNode parameters = (ArrayNode) cadidate.get(APPROVAL_PARAMETER);
                String positionCode = null;
                for (JsonNode parameter : parameters) {
                    if (parameter.get(APPROVAL_CODE_POSITION) != null) {
                        positionCode = parameter.get(APPROVAL_CODE_POSITION).textValue();
                        break;
                    }
                }
                if (StringUtils.isNotEmpty(positionCode)) {
                    IRequest iRequest = RequestHelper.newEmptyRequest();
                    iRequest.setLocale("zh_CN");
                    RequestHelper.setCurrentRequest(iRequest);
                    Position position = positionMapper.getPositionByCode(positionCode);
                    RequestHelper.clearCurrentRequest();
                    index.put(ASSIGNEE, position.getPositionId());
                    index.put(ASSIGNEE_CODE, position.getPositionCode());
                    index.put(ASSIGNEE_NAME, position.getName());
                }
            } else if (APPOINTED_EMPLOYEE.equalsIgnoreCase(ruleCode)) {
                String employeeCode = null;
                ArrayNode parameters = (ArrayNode) cadidate.get(APPROVAL_PARAMETER);
                for (JsonNode parameter : parameters) {
                    if (parameter.get(APPROVAL_CODE_EMPLOYEE) != null) {
                        employeeCode = parameter.get(APPROVAL_CODE_EMPLOYEE).textValue();
                        break;
                    }
                }
                if (StringUtils.isNotEmpty(employeeCode)) {
                    HlsEmployee employee = hlsEmployeeMapper.queryByCode(employeeCode);
                    index.put(ASSIGNEE, employee.getEmployeeCode());
                    index.put(ASSIGNEE_CODE, employee.getEmployeeCode());
                    index.put(ASSIGNEE_NAME, employee.getName());
                }
            }
            if (record != null) {
                index.put(DESCRIPTION, record.getDescription());
            }
            ArrayNode bussinessRuleNode = processBussinessRuleNode((ObjectNode) cadidate);
            if (bussinessRuleNode != null) {
                index.set(RULES, bussinessRuleNode);
            }
            assignmentNode.add(index);
        }
        ruleNode.set("assignment", assignmentNode);
        return ruleNode;
    }

    private ArrayNode processBussinessRuleNode(ObjectNode node) {
        ArrayNode businessRuleNode = null;
        ArrayNode businessRules = (ArrayNode) node.get(APPROVAL_BUSINESS_RULE);
        if (businessRules != null) {
            businessRuleNode = objectMapper.createArrayNode();
            for (JsonNode index : businessRules) {
                ObjectNode ruleNode = (ObjectNode) index;
                String ruleCode = ruleNode.get(CODE.toUpperCase()).textValue();
                BusinessRuleHeader rule = businessRuleHeaderMapper.selectByCode(ruleCode);
                if (rule != null) {
                    ObjectNode objectNode = objectMapper.createObjectNode();
                    objectNode.put(CODE, ruleCode);
                    objectNode.put("businessRuleCode", rule.getCode());
                    objectNode.put("businessRuleDescription", rule.getDescription());
                    businessRuleNode.add(objectNode);
                }
            }

        }
        return businessRuleNode;
    }

    // 根据code返回ApproveCandidateRule对象
    private ApproveCandidateRule getCandidateRule(String code) {
        ApproveCandidateRule record = new ApproveCandidateRule();
        record.setCode(code);
        record.setEnableFlag("Y");
        record = candidateRuleMapper.selectOne(record);
        return record;
    }

    private boolean hasPermission(JsonNode ruleMap, DelegateExecution execution) {
        if (ruleMap == null) {
            return true;
        }
        ExpressionManager expressionManager = processEngineConfiguration.getExpressionManager();

        // 取权限code
        ArrayNode rules = (ArrayNode) ruleMap.get(APPROVAL_BUSINESS_RULE);
        if (rules == null) {
            return true;
        }
        String code;
        BusinessRuleHeader header;
        BusinessRuleLine line = new BusinessRuleLine();
        List<BusinessRuleLine> lines;
        Date now = new Date();
        Date startDate, endDate;
        Object result;
        for (JsonNode rule : rules) {
            code = rule.get(CODE.toUpperCase()).textValue();
            if (StringUtils.isEmpty(code)) {
                continue;
            }
            /*header = businessRuleHeaderMapper.selectByCode(code);
            if (header == null) {
                logger.warn("businessRule is disable or not exist");
                continue;
            }
            startDate = header.getStartActiveDate();
            endDate = header.getEndActiveDate();
            if (startDate != null) {
                //验证是否已到启用日期
                if (now.before(startDate)) {
                    logger.warn("now before startDate ,businessRule is disable");
                    continue;
                }
            }
            if (endDate != null) {
                //验证是否已经过期
                if (now.after(endDate)) {
                    logger.warn("now after startDate ,businessRule is disable");
                    continue;
                }
            }
            line.setBusinessRuleId(header.getBusinessRuleId());
            line.setEnableFlag("Y");
            lines = businessRuleLineMapper.select(line);*/

            result = expressionManager.createExpression(code).getValue(execution);
            if (Boolean.FALSE.equals(result)) {
                return false;
            }

        }
        return true;
    }
}
