package com.hand.hap.activiti.components;

import com.hand.hap.activiti.custom.IActivitiBean;
import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import hls.core.cont.dto.*;
import hls.core.cont.mapper.*;
import hls.core.cont.service.ConContractChangeService;
import hls.core.cont.service.ConContractService;
import hls.core.csh.dto.*;
import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.csh.exception.ChangeLimitException;
import hls.core.csh.exception.WriteOffTypeNullException;
import hls.core.csh.mapper.HlsCusConContractCashflowMapper;
import hls.core.csh.mapper.HlsCusCshTransactionMapper;
import hls.core.csh.mapper.HlsCusTransactionRelationMapper;
import hls.core.csh.service.CshWriteOffService;
import hls.core.hls.components.CalculateUtil;
import hls.core.hls.components.DocumentChangeCommon;
import hls.core.hls.dto.HlsCusHlsDocumentVersionRel;
import hls.core.hls.dto.HlsDocumentVersionRel;
import hls.core.hls.mapper.HlsCusHlsDocumentVersionRelMapper;
import hls.core.prj.dto.HlsCusHlsDocumentVersion;
import hls.core.prj.mapper.HlsCusHlsDocumentVersionMapper;
import hls.core.sys.dto.SysUser;
import hls.core.sys.service.SysUserService;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.JavaDelegate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.InvocationTargetException;
import java.util.*;
import java.util.function.Predicate;
import java.util.stream.Collectors;

/**
 * Created by Administrator on 2017/4/18.
 */
@Component
public class ContractChangeServiceTask implements JavaDelegate, IActivitiBean {

    public static final long ITEM = 1L;//租金
    public static final String WRITE_OFF_FLAG_FULL = "FULL";//现金流完全核销
    public static final String APPROVED = "APPROVED";//审批通过
    public static final String DEPOSIT = "DEPOSIT";//保证金
    public static final String INFLOW = "INFLOW";//现金流方向-流入
    public static final String RECEIPT_CREDIT = "RECEIPT_CREDIT";//收款核销债权
    public static final String DEPOSIT_CREDIT = "DEPOSIT_CREDIT";//保证金核销债权
    public static final String REVERSED_FLAG_N = "N";//未反冲
    public static final String PARTIAL = "PARTIAL";//部分核销
    public static final String FULL = "FULL";//完全核销
    public static final String ET = "ET";//合同提前结清
    public static final String CHANGE_REQ = "CHANGE_REQ";
    public static final String CON_CONTRACT = "con_contract";
    public static final String CON_CONTRACT_CASHFLOW = "con_contract_cashflow";
    public static final String CON_CONTRACT_GUARANTOR = "con_contract_guarantor";
    public static final String CON_CONTRACT_LEASE_ITEM = "con_contract_lease_item";
    public static final String CON_CONTRACT_MORTGAGE = "con_contract_mortgage";
    public static final String CON_CONTRACT_TENANT = "con_contract_tenant";
    public static final String INCEPT = "INCEPT";
    @Autowired
    private ConContractService contractService;
    @Autowired
    private SysUserService userService;
    @Autowired
    private ConContractChangeService contractChangeService;
    @Autowired
    private DatabaseLockProvider databaseLockProvider;
    @Autowired
    private HlsCusCshTransactionMapper transactionMapper;
    @Autowired
    private CshWriteOffService writeOffService;
    @Autowired
    private HlsCusTransactionRelationMapper transactionRelationMapper;
    @Autowired
    private HlsCusHlsDocumentVersionMapper documentVersionMapper;
    @Autowired
    private HlsCusHlsDocumentVersionRelMapper documentVersionRelMapper;
    @Autowired
    private DocumentChangeCommon documentChangeCommon;
    @Autowired
    private HlsCusConContractMapper conContractMapper;
    @Autowired
    private HlsCusConContractHMapper conContractHMapper;
    @Autowired
    private HlsCusConContractTenantMapper conContractTenantMapper;
    @Autowired
    private HlsCusConContractTenantHMapper conContractTenantHMapper;
    @Autowired
    private HlsCusConContractMortgageMapper conContractMortgageMapper;
    @Autowired
    private HlsCusConContractMortgageHMapper conContractMortgageHMapper;
    @Autowired
    private HlsCusConContractGuarantorMapper conContractGuarantorMapper;
    @Autowired
    private HlsCusConContractGuarantorHMapper conContractGuarantorHMapper;
    @Autowired
    private HlsCusConContractLeaseItemMapper conContractLeaseItemMapper;
    @Autowired
    private HlsCusConContractLeaseItemHMapper conContractLeaseItemHMapper;
    @Autowired
    private HlsCusConContractCashflowHMapper cashflowHMapper;
    @Autowired
    private HlsCusConContractCashflowMapper conContractCashflowMapper;


    @Override
    public void execute(DelegateExecution delegateExecution) {
        String flag = null;
        IRequest requestCtx = (IRequest) delegateExecution.getVariable("iRequest");
        String result = (String) delegateExecution.getVariable("approveResult");
        HlsCusConContractChangeReq contractChangeReq = (HlsCusConContractChangeReq) delegateExecution.getVariable("contractChangeReq");
        String employeeCode = (String) delegateExecution.getVariable("startUserName");
        HlsCusCshConContract conContract = (HlsCusCshConContract) delegateExecution.getVariable("conContract");
        String earlyTerminationFlag = (String) delegateExecution.getVariable("earlyTerminationFlag");
        //lock con_contract
        databaseLockProvider.lock(conContract);
        SysUser sysUser = userService.queryUserByCode(employeeCode);
        if (sysUser != null) {
            Long userId = Long.valueOf(sysUser.getUserId());
            requestCtx.setUserId(userId);
        }

        if ("APPROVED".equalsIgnoreCase(result)) {
            flag = "APPROVED";
        } else if ("CANCEL".equalsIgnoreCase(result)) {
            flag = "CANCEL";
        } else {
            flag = "REJECTED";
        }

        HlsCusConContractChangeReq cccr = new HlsCusConContractChangeReq();
        cccr.setChangeReqId(contractChangeReq.getChangeReqId());
        cccr.setProcessInstanceId(Long.parseLong(delegateExecution.getProcessInstanceId()));
        contractChangeService.updateByPrimaryKeySelective(requestCtx, cccr);

        if (APPROVED.equalsIgnoreCase(flag)) {
            //变更通过后，插入hls_document_version（类型为HISTORY
            HlsCusHlsDocumentVersion hisDocumentVersion = documentChangeCommon.afterChangeRequest(requestCtx, contractChangeReq);

            HlsCusHlsDocumentVersion hlsDocumentVersion = new HlsCusHlsDocumentVersion();
            hlsDocumentVersion.setSourceDocumentId(contractChangeReq.getChangeReqId());
            hlsDocumentVersion.setVersionType(CHANGE_REQ);
            hlsDocumentVersion.setSourceDocumentCategory(DocumentChangeCommon.getTableName(contractChangeReq.getClass()));
            hlsDocumentVersion = documentVersionMapper.selectOne(hlsDocumentVersion);
            Long versionId = hlsDocumentVersion.getVersionId();
            HlsCusHlsDocumentVersionRel hlsDocumentVersionRel = new HlsCusHlsDocumentVersionRel();
            hlsDocumentVersionRel.setVersionId(versionId);
            List<HlsCusHlsDocumentVersionRel> documentVersionRels = documentVersionRelMapper.select(hlsDocumentVersionRel);

            List<HlsDocumentVersionRel> contractList = getFilterDocumentVersionRel(documentVersionRels, item -> CON_CONTRACT.equalsIgnoreCase(item.getFromTable()));
            List<HlsDocumentVersionRel> contractCashflowList = getFilterDocumentVersionRel(documentVersionRels, item -> CON_CONTRACT_CASHFLOW.equalsIgnoreCase(item.getFromTable()));
            List<HlsDocumentVersionRel> contractGuarantorList = getFilterDocumentVersionRel(documentVersionRels, item -> CON_CONTRACT_GUARANTOR.equalsIgnoreCase(item.getFromTable()));
            List<HlsDocumentVersionRel> contractLeaseItemList = getFilterDocumentVersionRel(documentVersionRels, item -> CON_CONTRACT_LEASE_ITEM.equalsIgnoreCase(item.getFromTable()));
            List<HlsDocumentVersionRel> contractMortgageList = getFilterDocumentVersionRel(documentVersionRels, item -> CON_CONTRACT_MORTGAGE.equalsIgnoreCase(item.getFromTable()));
            List<HlsDocumentVersionRel> contracttenantList = getFilterDocumentVersionRel(documentVersionRels, item -> CON_CONTRACT_TENANT.equalsIgnoreCase(item.getFromTable()));

            //复制整套合同数据
            Map param = new HashMap<>();
            contractList.forEach(item -> {
                Long fromPk = item.getFromPk();
                HlsCusConContract cusConContract = conContractMapper.selectByPrimaryKey(fromPk);
                ConContractH conContractH = documentChangeCommon.copyHistoryCommon(requestCtx, cusConContract, null, hisDocumentVersion.getVersionId());
                param.put(DocumentChangeCommon.FIX_COLUMN_NAME, "contractId");
                param.put(DocumentChangeCommon.FIX_COLUMN_ID, conContractH.getContractId());
            });


            contractCashflowList.forEach(item -> {
                Long fromPk = item.getFromPk();
                HlsCusConContractCashflow hlsCusConContractCashflow = conContractCashflowMapper.selectByPrimaryKey(fromPk);
                documentChangeCommon.copyHistoryCommon(requestCtx, hlsCusConContractCashflow, param, hisDocumentVersion.getVersionId());
            });
            contractGuarantorList.forEach(item -> {
                Long fromPk = item.getFromPk();
                HlsCusConContractGuarantor hlsCusConContractGuarantor = conContractGuarantorMapper.selectByPrimaryKey(fromPk);
                documentChangeCommon.copyHistoryCommon(requestCtx, hlsCusConContractGuarantor, param, hisDocumentVersion.getVersionId());
            });
            contractLeaseItemList.forEach(item -> {
                Long fromPk = item.getFromPk();
                HlsCusConContractLeaseItem hlsCusConContractLeaseItem = conContractLeaseItemMapper.selectByPrimaryKey(fromPk);
                documentChangeCommon.copyHistoryCommon(requestCtx, hlsCusConContractLeaseItem, param, hisDocumentVersion.getVersionId());
            });
            contractMortgageList.forEach(item -> {
                Long fromPk = item.getFromPk();
                HlsCusConContractMortgage hlsCusConContractMortgage = conContractMortgageMapper.selectByPrimaryKey(fromPk);
                documentChangeCommon.copyHistoryCommon(requestCtx, hlsCusConContractMortgage, param, hisDocumentVersion.getVersionId());
            });
            contracttenantList.forEach(item -> {
                Long fromPk = item.getFromPk();
                HlsCusConContractTenant hlsCusConContractTenant = conContractTenantMapper.selectByPrimaryKey(fromPk);
                documentChangeCommon.copyHistoryCommon(requestCtx, hlsCusConContractTenant, param, hisDocumentVersion.getVersionId());
            });


            //用change req对应的version_id，从history数据中，覆盖回原合同表(整套)
            HlsDocumentVersionRel contractRel = contractList.get(0);
            Long fromPk = contractRel.getFromPk();
            Long toPk = contractRel.getToPk();

            HlsCusConContract cusConContract = conContractMapper.selectByPrimaryKey(fromPk);
            HlsCusConContractH cusConContractH = conContractHMapper.selectByPrimaryKey(toPk);
            cusConContract = documentChangeCommon.recoverHistoryCommon(requestCtx, cusConContract, cusConContractH, null);

            Map map = new HashMap<>();
            map.put(DocumentChangeCommon.FIX_COLUMN_NAME, "contractId");
            map.put(DocumentChangeCommon.FIX_COLUMN_ID, cusConContract.getContractId());


            HlsCusConContractCashflowH cashflowH = new HlsCusConContractCashflowH();
            cashflowH.setContractId(cusConContractH.getContractId());
            List<HlsCusConContractCashflowH> cashflowHList = cashflowHMapper.select(cashflowH);
            for (HlsCusConContractCashflowH hlsCusConContractCashflowH : cashflowHList) {
                HlsCusConContractCashflow hlsCusConContractCashflow = null;
                for (HlsDocumentVersionRel child : contractCashflowList) {
                    if (child.getToPk().compareTo(hlsCusConContractCashflowH.getCashflowId()) == 0) {
                        hlsCusConContractCashflow = conContractCashflowMapper.selectByPrimaryKey(child.getFromPk());
                        break;
                    } else {
                        hlsCusConContractCashflow = new HlsCusConContractCashflow();
                    }
                }
                documentChangeCommon.recoverHistoryCommon(requestCtx, hlsCusConContractCashflow, hlsCusConContractCashflowH, map);
            }

            HlsCusConContractGuarantorH guarantorH = new HlsCusConContractGuarantorH();
            guarantorH.setContractId(cusConContractH.getContractId());
            List<HlsCusConContractGuarantorH> guarantorHList = conContractGuarantorHMapper.select(guarantorH);
            for (HlsCusConContractGuarantorH hlsCusConContractGuarantorH : guarantorHList) {
                HlsCusConContractGuarantor hlsCusConContractGuarantor = null;
                for (HlsDocumentVersionRel child : contractGuarantorList) {
                    if (child.getToPk().compareTo(hlsCusConContractGuarantorH.getContractGuarantorId()) == 0) {
                        hlsCusConContractGuarantor = conContractGuarantorMapper.selectByPrimaryKey(child.getFromPk());
                        break;
                    } else {
                        hlsCusConContractGuarantor = new HlsCusConContractGuarantor();
                    }
                }
                documentChangeCommon.recoverHistoryCommon(requestCtx, hlsCusConContractGuarantor, hlsCusConContractGuarantorH, map);
            }

            HlsCusConContractLeaseItemH leaseItemH = new HlsCusConContractLeaseItemH();
            leaseItemH.setContractId(cusConContractH.getContractId());
            List<HlsCusConContractLeaseItemH> leaseItemHList = conContractLeaseItemHMapper.select(leaseItemH);
            for (HlsCusConContractLeaseItemH hlsCusConContractLeaseItemH : leaseItemHList) {
                HlsCusConContractLeaseItem hlsCusConContractLeaseItem = null;
                for (HlsDocumentVersionRel child : contractLeaseItemList) {
                    if (child.getToPk().compareTo(hlsCusConContractLeaseItemH.getContractLeaseItemId()) == 0) {
                        hlsCusConContractLeaseItem = conContractLeaseItemMapper.selectByPrimaryKey(child.getFromPk());
                        break;
                    } else {
                        hlsCusConContractLeaseItem = new HlsCusConContractLeaseItem();
                    }
                }
                documentChangeCommon.recoverHistoryCommon(requestCtx, hlsCusConContractLeaseItem, hlsCusConContractLeaseItemH, map);
            }

            HlsCusConContractMortgageH mortgageH = new HlsCusConContractMortgageH();
            mortgageH.setContractId(cusConContractH.getContractId());
            List<HlsCusConContractMortgageH> mortgageHList = conContractMortgageHMapper.select(mortgageH);
            for (HlsCusConContractMortgageH hlsCusConContractMortgageH : mortgageHList) {
                HlsCusConContractMortgage hlsCusConContractMortgage = null;
                for (HlsDocumentVersionRel child : contractMortgageList) {
                    if (child.getToPk().compareTo(hlsCusConContractMortgageH.getContractMortgageId()) == 0) {
                        hlsCusConContractMortgage = conContractMortgageMapper.selectByPrimaryKey(child.getFromPk());
                        break;
                    } else {
                        hlsCusConContractMortgage = new HlsCusConContractMortgage();
                    }
                }
                documentChangeCommon.recoverHistoryCommon(requestCtx, hlsCusConContractMortgage, hlsCusConContractMortgageH, map);
            }

            HlsCusConContractTenantH tenantH = new HlsCusConContractTenantH();
            tenantH.setContractId(cusConContractH.getContractId());
            List<HlsCusConContractTenantH> tenantHList = conContractTenantHMapper.select(tenantH);
            for (HlsCusConContractTenantH hlsCusConContractTenantH : tenantHList) {
                HlsCusConContractTenant hlsCusConContractTenant = null;
                for (HlsDocumentVersionRel child : contracttenantList) {
                    if (child.getToPk().compareTo(hlsCusConContractTenantH.getContractTenantId()) == 0) {
                        hlsCusConContractTenant = conContractTenantMapper.selectByPrimaryKey(child.getFromPk());
                        break;
                    } else {
                        hlsCusConContractTenant = new HlsCusConContractTenant();
                    }
                }
                documentChangeCommon.recoverHistoryCommon(requestCtx, hlsCusConContractTenant, hlsCusConContractTenantH, map);
            }

        } else {
            try {
                contractChangeService.examineAndApprove(requestCtx, contractChangeReq.getChangeReqId(), contractChangeReq.getContractId(), flag);
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            } catch (NoSuchMethodException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            } catch (InvocationTargetException e) {
                e.printStackTrace();
            }
            Long contractId = contractChangeReq.getContractId();
            HlsCusConContract contract = new HlsCusConContract();
            contract.setContractId(contractId);
            contract.setContractStatus(INCEPT);
            contractService.updateByPrimaryKeySelective(requestCtx, contract);//当提前结清的合同被拒绝时,应当将合同重新变回起租

        }

        if (APPROVED.equalsIgnoreCase(flag) && "Y".equalsIgnoreCase(earlyTerminationFlag)) {//当为提前结清,核销所有的未核销的现金流
            HlsCusConContractCashflow cashflow = new HlsCusConContractCashflow();
            cashflow.setContract_id(contractChangeReq.getContractId());
            cashflow.setCf_direction(INFLOW);
            List<HlsCusConContractCashflow> cashflows = conContractCashflowMapper.select(cashflow);
            cashflows = cashflows.stream().filter(item -> !"BLOCK".equalsIgnoreCase(item.getCf_status()) && item.getCf_item().intValue() != 2 && item.getCf_type().intValue() != 2 && item.getCf_type().intValue() != 5 && item.getCf_type().intValue() != 5).collect(Collectors.toList());
            HlsCusCshTransaction transaction = new HlsCusCshTransaction();
            transaction.setContractId(cashflow.getContract_id());
            transaction.setTransactionType(DEPOSIT);
            List<HlsCusCshTransaction> depositTransactions = transactionMapper.select(transaction);//获取保证金类型的现金事务信息
            HlsCusTransactionRelation hlsCusTransactionRelation = new HlsCusTransactionRelation();
            hlsCusTransactionRelation.setRelationDocId(contractChangeReq.getChangeReqId());
            hlsCusTransactionRelation.setRelationDocCategory(DocumentChangeCommon.getTableName(contractChangeReq.getClass()));
            List<HlsCusTransactionRelation> transactionRelations = transactionRelationMapper.select(hlsCusTransactionRelation);//找出关联的现金事物
            List<HlsCusCshTransaction> hlsCusCshTransactions = new ArrayList<>();//现金事物集合
            transactionRelations.forEach(item -> {
                HlsCusCshTransaction hlsCusCshTransaction = transactionMapper.selectByPrimaryKey(item.getTransactionId());
                hlsCusCshTransactions.add(hlsCusCshTransaction);
            });

            List<Map> params = new ArrayList<>();//记录现金事物和对应的剩余金额
            hlsCusCshTransactions.addAll(depositTransactions);//合并两个list
            hlsCusCshTransactions.forEach(item -> {
                Map param = new HashMap();
                param.put("leftAmount", CalculateUtil.sub(item.getTransactionAmount(), item.getWriteOffAmount()));
                param.put("transactionId", item.getTransactionId());
                if (DEPOSIT.equalsIgnoreCase(item.getTransactionType())) {
                    param.put("writeOffType", DEPOSIT_CREDIT);
                } else {
                    param.put("writeOffType", RECEIPT_CREDIT);
                }
                params.add(param);
            });


            cashflows = cashflows.stream().filter(item -> !WRITE_OFF_FLAG_FULL.equalsIgnoreCase(item.getWrite_off_flag())).collect(Collectors.toList());
            List<HlsCusCshWriteOff> cshWriteOffs = autoGroupingWriteOff(params, cashflows);
            try {
                writeOffService.updateWriteOff(requestCtx, cshWriteOffs, requestCtx.getCompanyId());
            } catch (BeyondAmountLimitException e) {
                e.printStackTrace();
            } catch (WriteOffTypeNullException e) {
                e.printStackTrace();
            } catch (ChangeLimitException e) {
                e.printStackTrace();
            }

            Long contractId = contractChangeReq.getContractId();
            HlsCusConContract cusConContract = new HlsCusConContract();
            cusConContract.setContractId(contractId);
            cusConContract.setContractStatus(ET);
            contractService.updateByPrimaryKeySelective(requestCtx, cusConContract);
        }
        //变更状态进入审批通过
        contractChangeService.modifyStateReq(requestCtx, contractChangeReq, flag);

    }

    /**
     * 自动核销分组,返回一个核销的集合
     *
     * @param params
     * @param cashflows
     * @return
     */
    private List<HlsCusCshWriteOff> autoGroupingWriteOff(List<Map> params, List<HlsCusConContractCashflow> cashflows) {
        List<HlsCusCshWriteOff> writeOffs = new ArrayList<>();
        params.forEach(item -> groupWriteOffRecord(cashflows, writeOffs, item));
        return writeOffs;
    }

    /**
     * 获取核销需要的数据
     *
     * @param item
     * @param writeOffType
     * @return
     */
    private static HlsCusCshWriteOff getWriteOffObj(HlsCusConContractCashflow item, String writeOffType) {
        HlsCusCshWriteOff writeOff = new HlsCusCshWriteOff();
        writeOff.setWriteOffType(writeOffType);
        writeOff.setWriteOffDate(new Date());
        //writeOff.setTransactionId(transactionId);
        writeOff.setCashflowId(item.getCashflow_id());
        writeOff.setContractId(item.getContract_id());
        writeOff.setTimes(item.getTimes());
        writeOff.setCfItem(item.getCf_item());
        writeOff.setCfType(item.getCf_type());
        writeOff.setCshWriteOffAmount(CalculateUtil.sub(item.getDue_amount(), item.getReceived_amount()));
        writeOff.setWriteOffDueAmount(item.getDue_amount());
        writeOff.setWriteOffPrincipal(CalculateUtil.sub(item.getPrincipal(), item.getReceived_principal()));
        writeOff.setWriteOffInterest(CalculateUtil.sub(item.getInterest(), item.getReceived_interest()));
        writeOff.setReversedFlag(REVERSED_FLAG_N);
        return writeOff;
    }

    /**
     * 自动核销分组
     *
     * @param cashflows
     * @param writeOffs
     * @param param
     */
    private void groupWriteOffRecord(List<HlsCusConContractCashflow> cashflows, List<HlsCusCshWriteOff> writeOffs, Map param) {
        cashflows.forEach(flow -> {
            Double leftAmount = (Double) param.get("leftAmount");//当前现金事物所剩未被核销的金额
            Long transactionId = (Long) param.get("transactionId");//现金事物ID
            if (!FULL.equalsIgnoreCase(flow.getWrite_off_flag()) && leftAmount.compareTo(0D) > 0) {
                if (leftAmount.compareTo(CalculateUtil.sub(flow.getDue_amount(), flow.getReceived_amount())) == 1) {//当现金事物上的钱足够核销当前现金流,且还有结余时
                    HlsCusCshWriteOff writeOffObj = getWriteOffObj(flow, (String) param.get("writeOffType"));
                    writeOffObj.setTransactionId(transactionId);
                    writeOffObj.setCshTransactionId(transactionId);
                    writeOffs.add(writeOffObj);
                    param.put("leftAmount", CalculateUtil.sub(leftAmount, CalculateUtil.sub(flow.getDue_amount(), flow.getReceived_amount())));
                    flow.setWrite_off_flag(FULL);
                } else if (leftAmount.compareTo(CalculateUtil.sub(flow.getDue_amount(), flow.getReceived_amount())) == -1) {//当现金事物上的钱不完全够核销时
                    flow.setReceived_amount(CalculateUtil.add(flow.getReceived_amount(),leftAmount));//当现金事物上余额不足是,应该把当前所剩的钱既为该核销记录的应收金额
                    HlsCusCshWriteOff writeOffObj = getWriteOffObj(flow, (String) param.get("writeOffType"));
                    writeOffObj.setTransactionId(transactionId);
                    writeOffObj.setCshTransactionId(transactionId);
                    writeOffs.add(writeOffObj);
                    param.put("leftAmount", 0D);
                    flow.setWrite_off_flag(PARTIAL);
                }
            }
        });
    }

    private List<HlsDocumentVersionRel> getFilterDocumentVersionRel(List<HlsCusHlsDocumentVersionRel> list, Predicate<HlsCusHlsDocumentVersionRel> predicate) {
        return list.stream().filter(predicate).collect(Collectors.toList());
    }
}
