package com.hand.hap.activiti.components;

import com.alibaba.fastjson.JSON;
import com.hand.hap.activiti.custom.IActivitiBean;
import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import hls.core.fct.dto.HlsCusFctContractWithdraw;
import hls.core.fct.dto.HlsCusFctContractWithdrawAprl;
import hls.core.fct.service.FctContractWithdrawAprlService;
import hls.core.fct.service.FctContractWithdrawService;
import hls.core.sys.dto.SysUser;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.service.SysUserService;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.JavaDelegate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by SYJ on 2017/12/7
 * 预计收款安排制作 审批结束
 */

@Component
public class FctAdvancesReceivedServiceTask implements JavaDelegate, IActivitiBean {


    @Autowired
    private FctContractWithdrawService fctContractWithdrawService;
    @Autowired
    private SysUserService userService;

    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private DatabaseLockProvider databaseLockProvider;
    @Autowired
    private FctContractWithdrawAprlService fctContractWithdrawAprlService;


    public static final String PROPERTY_FCT_ADVANCES_RECEIVED_APPROVED = "FCT_ADVANCES_RECEIVED.APPROVED";
    public static final String PROPERTY_FCT_ADVANCES_RECEIVED_REJECT = "FCT_ADVANCES_RECEIVED.REJECT";
    public static final String PROPERTY_FCT_ADVANCES_RECEIVED_CANCEL = "FCT_ADVANCES_RECEIVED.CANCEL";

    public FctAdvancesReceivedServiceTask() {
    }

    @Override
    public void execute(DelegateExecution delegateExecution) {
        String flag = null;
        String flagDesc = null;
        String evenCode = null;
        IRequest requestCtx = (IRequest) delegateExecution.getVariable("iRequest");
        String result = (String) delegateExecution.getVariable("approveResult");
        String hlsCusFctContractWithdraw = (String) delegateExecution.getVariable("hlsCusFctContractWithdraw");
        HlsCusFctContractWithdraw fctContractWithdraw = JSON.parseObject(hlsCusFctContractWithdraw, HlsCusFctContractWithdraw.class);
        String employeeCode = (String) delegateExecution.getVariable("startUserName");
        SysUser sysUser = userService.queryUserByCode(employeeCode);

        HlsCusFctContractWithdraw withdraw = new HlsCusFctContractWithdraw();
        withdraw.setWithdrawId(fctContractWithdraw.getWithdrawId());
        withdraw = fctContractWithdrawService.selectByPrimaryKey(requestCtx, withdraw);
        // lock con_contract
        databaseLockProvider.lock(withdraw);
        if ("APPROVED".equalsIgnoreCase(result)) {
            flag = "APPROVED";
            flagDesc = "通过";
            evenCode = PROPERTY_FCT_ADVANCES_RECEIVED_APPROVED;
            withdraw.setWithdrawStatus("ADVANCES_RECEIVED");
            fctContractWithdrawService.updateByPrimaryKeySelective(requestCtx, withdraw);
        } else if ("CANCEL".equalsIgnoreCase(result)) {
            flag = "CANCEL";
            flagDesc = "取消";
            evenCode = PROPERTY_FCT_ADVANCES_RECEIVED_CANCEL;
        } else {
            flag = "REJECTED";
            flagDesc = "拒绝";
            withdraw.setWithdrawStatus("WITHDRAW_REQUEST");
            evenCode = PROPERTY_FCT_ADVANCES_RECEIVED_REJECT;
            fctContractWithdrawService.updateByPrimaryKeySelective(requestCtx, withdraw);
        }

        Long withdrawApprovalId = (Long) delegateExecution.getVariable("withdrawApprovalId");

        //更新授信审批表
        HlsCusFctContractWithdrawAprl hlsCusFctContractWithdrawAprl = new HlsCusFctContractWithdrawAprl();
        hlsCusFctContractWithdrawAprl.setWithdrawApprovalId(withdrawApprovalId);
        hlsCusFctContractWithdrawAprl.setStatus(flag);
        hlsCusFctContractWithdrawAprl.setProcessInstanceId(Long.parseLong(delegateExecution.getProcessInstanceId()));
        fctContractWithdrawAprlService.updateByPrimaryKeySelective(requestCtx, hlsCusFctContractWithdrawAprl);

        Map<String, Object> evenParams = new HashMap<>();
        evenParams.put("message", "保理提款" + withdraw.getWithdrawName() + "预计收款安排制作审批" + flagDesc);
        evenParams.put("noticeTitle", "保理提款预计收款安排制作审批");
        evenParams.put("url", "");
        evenParams.put("level", 1L);
        evenParams.put("noticeType", "NOTICE");
        sysEventService.eventSave(requestCtx, withdraw.getWithdrawId(), withdraw.getDocumentCategory(), withdraw.getDocumentType(), "FCT", evenCode, "P2D", evenParams);
    }
}
