package com.hand.hap.activiti.components;

import com.alibaba.fastjson.JSON;
import com.hand.hap.activiti.custom.IActivitiBean;
import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import hls.core.fct.dto.*;
import hls.core.fct.mapper.HlsCusFctProjectAttachmentMapper;
import hls.core.fct.mapper.HlsCusFctProjectConSignAttMapper;
import hls.core.fct.service.*;
import hls.core.sys.dto.SysUser;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.service.SysUserService;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.JavaDelegate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * Created by SYJ on 2017/11/16.
 */
@Component
public class FctContractSignServiceTask implements JavaDelegate, IActivitiBean {


    @Autowired
    private FctProjectService fctProjectService;
    @Autowired
    private SysUserService userService;

    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private DatabaseLockProvider databaseLockProvider;

    @Autowired
    private FctProjectConSignAprlService fctProjectConSignAprlService;
    @Autowired
    private HlsCusFctProjectConSignAttMapper fctProjectConSignAttMapper;
    @Autowired
    private FctProjectConSignAttService fctProjectConSignAttService;
    @Autowired
    private FctProjectAttachmentService fctProjectAttachmentService;
    @Autowired
    private FctContractService fctContractService;
    @Autowired
    private HlsCusFctProjectAttachmentMapper fctProjectAttachmentMapper;

    public static final String PROPERTY_FCT_CONTRACT_SIGN_SUBMIT = "FCT_PROJECT.SIGN.SUBMIT";

    public FctContractSignServiceTask() {
    }

    @Override
    public void execute(DelegateExecution delegateExecution) {
        String flag = null;
        String flagDesc = null;
        IRequest requestCtx = (IRequest) delegateExecution.getVariable("iRequest");
        String result = (String) delegateExecution.getVariable("approveResult");
        String project = (String) delegateExecution.getVariable("fctProject");

        Date signDate = (Date) delegateExecution.getVariable("signDate");
        String signDescription = (String) delegateExecution.getVariable("signDescription");

        HlsCusFctProject fctproject = JSON.parseObject(project, HlsCusFctProject.class);

        String employeeCode = (String) delegateExecution.getVariable("startUserName");
        SysUser sysUser = userService.queryUserByCode(employeeCode);
        HlsCusFctProject cc = new HlsCusFctProject();
        cc.setProjectId(fctproject.getProjectId());
        cc = fctProjectService.selectByPrimaryKey(requestCtx, cc);
        // lock con_contract
        databaseLockProvider.lock(cc);
        List<HlsCusFctContract> fctContractList = new ArrayList<HlsCusFctContract>();

        Long signApprovalId = (Long) delegateExecution.getVariable("signApprovalId");


        if ("APPROVED".equalsIgnoreCase(result)) {
            flag = "APPROVED";
            flagDesc = "通过";

            /*只要有合同状态为GENERATED 则PROCESS_IN_SIGN*/
            HlsCusFctProjectAttachment fctProjectAttachment = new HlsCusFctProjectAttachment();
            Long projectId = fctproject.getProjectId();
            fctProjectAttachment.setProjectId(projectId);
            fctProjectAttachment.setStatus("GENERATED");
            List<HlsCusFctProjectAttachment> hlsCusFctProjectAttachmentLists = new ArrayList<>();
            hlsCusFctProjectAttachmentLists = fctProjectAttachmentMapper.fctProjectAttachmentForSignProcessQuery(fctProjectAttachment);
            if(hlsCusFctProjectAttachmentLists.size()>0){
                cc.setContractStatus("PROCESS_IN_SIGN");
            }else{
                cc.setContractStatus("SIGN");
            }
            cc.setSignApprovalId(signApprovalId);
            cc.setSignDescription(signDescription);
            cc.setSignDate(signDate);
            fctProjectService.updateByPrimaryKeySelective(requestCtx, cc);
            fctContractList = fctContractService.createContractFromProject(requestCtx, cc);

            /*更新合同状态*/
            HlsCusFctProjectAttachment dto = new HlsCusFctProjectAttachment();
            dto.setProjectId(projectId);
            dto.setSignApprovalId(signApprovalId);
            List<HlsCusFctProjectAttachment> hlsCusFctProjectAttachmentList = new ArrayList<HlsCusFctProjectAttachment>();
            hlsCusFctProjectAttachmentList = fctProjectAttachmentMapper.fctContractContentHistoryQuery(dto);
            for (int i = 0; i < hlsCusFctProjectAttachmentList.size(); i++) {
                hlsCusFctProjectAttachmentList.get(i).setStatus("SIGNED");
                hlsCusFctProjectAttachmentList.get(i).set__status("update");
            }
            fctProjectAttachmentService.batchUpdate(requestCtx, hlsCusFctProjectAttachmentList);

        } else if ("CANCEL".equalsIgnoreCase(result)) {
            flag = "CANCEL";
            flagDesc = "取消";
        } else {
            flag = "REJECTED";
            flagDesc = "拒绝";
            cc.setContractStatus("UN_SIGN");
            fctProjectService.updateByPrimaryKeySelective(requestCtx, cc);

            /*更新合同状态*/
            HlsCusFctProjectAttachment dto = new HlsCusFctProjectAttachment();
            Long projectId = fctproject.getProjectId();
            dto.setProjectId(projectId);
            dto.setSignApprovalId(signApprovalId);
            List<HlsCusFctProjectAttachment> hlsCusFctProjectAttachmentList = new ArrayList<>();
            hlsCusFctProjectAttachmentList = fctProjectAttachmentMapper.fctContractContentHistoryQuery(dto);
            for (int i = 0; i < hlsCusFctProjectAttachmentList.size(); i++) {
                hlsCusFctProjectAttachmentList.get(i).setStatus("GENERATED");
                hlsCusFctProjectAttachmentList.get(i).set__status("update");
            }
            fctProjectAttachmentService.batchUpdate(requestCtx, hlsCusFctProjectAttachmentList);
        }

        //更新审批表
        HlsCusFctProjectConSignAprl fctProjectConSignAprl = new HlsCusFctProjectConSignAprl();
        fctProjectConSignAprl.setSignApprovalId(signApprovalId);
        fctProjectConSignAprl.setProjectId(cc.getProjectId());
        fctProjectConSignAprl.setStatus(flag);
        fctProjectConSignAprl.setSignDate(signDate);
            /*if(fctContractList.size()>0){
                fctProjectConSignAprl.setFctContractId(fctContractList.get(j).getContractId());
            }*/
        fctProjectConSignAprl.setSignDescription(signDescription);
        fctProjectConSignAprl.setProcessInstanceId(Long.parseLong(delegateExecution.getProcessInstanceId()));
        fctProjectConSignAprlService.updateByPrimaryKeySelective(requestCtx, fctProjectConSignAprl);

        //插入审批文件表
        HlsCusFctProjectAttachment fctProjectAttachment = new HlsCusFctProjectAttachment();
        List<HlsCusFctProjectAttachment> fctProjectAttachmentList = new ArrayList<HlsCusFctProjectAttachment>();
        fctProjectAttachment.setProjectId(fctproject.getProjectId());
        fctProjectAttachment.setSignApprovalId(signApprovalId);
        fctProjectAttachmentList = fctProjectAttachmentMapper.fctContractContentHistoryQuery(fctProjectAttachment);

        /*先删除签约审批文件明细表*/
        HlsCusFctProjectConSignAtt att = new HlsCusFctProjectConSignAtt();
        att.setSignApprovalId(signApprovalId);
        fctProjectConSignAttMapper.fctProjectConSignAttDelete(att);

        fctProjectAttachment.setProjectId(cc.getProjectId());
        fctProjectAttachment.setSignApprovalId(signApprovalId);
        if (flag == "APPROVED") {
            for (int i = 0; i < fctProjectAttachmentList.size(); i++) {
                HlsCusFctProjectConSignAtt fctProjectConSignAtt = new HlsCusFctProjectConSignAtt();
                fctProjectConSignAtt.setSignApprovalId(fctProjectConSignAprl.getSignApprovalId());
                fctProjectConSignAtt.setProjectAttachmentId(fctProjectAttachmentList.get(i).getProjectAttachmentId());
                fctProjectConSignAtt.setStatus(flag);
                for (int j = 0; j < fctContractList.size(); j++) {
                    if (fctProjectAttachmentList.get(i).getProjectAttachmentId().longValue() == fctContractList.get(j).getProjectAttachmentId().longValue()) {
                        fctProjectConSignAtt.setFctContractId(fctContractList.get(j).getContractId());
                    }
                }
                fctProjectConSignAtt.setProcessInstanceId(Long.parseLong(delegateExecution.getProcessInstanceId()));
                fctProjectConSignAttService.insertSelective(requestCtx, fctProjectConSignAtt);
            }
        }else if(flag == "REJECTED"){
            for (int i = 0; i < fctProjectAttachmentList.size(); i++) {
            HlsCusFctProjectConSignAtt fctProjectConSignAtt = new HlsCusFctProjectConSignAtt();
            fctProjectConSignAtt.setSignApprovalId(fctProjectConSignAprl.getSignApprovalId());
            fctProjectConSignAtt.setProjectAttachmentId(fctProjectAttachmentList.get(i).getProjectAttachmentId());
            fctProjectConSignAtt.setStatus(flag);
            fctProjectConSignAtt.setProcessInstanceId(Long.parseLong(delegateExecution.getProcessInstanceId()));
            fctProjectConSignAttService.insertSelective(requestCtx, fctProjectConSignAtt);
            }
        }

        Map<String, Object> evenParams = new HashMap<>();
        evenParams.put("message", "保理项目" + cc.getProjectNumber() + "签约审批" + flagDesc);
        evenParams.put("noticeTitle", "签约审批");
        evenParams.put("url", "");
        evenParams.put("level", 1L);
        evenParams.put("noticeType", "NOTICE");
        sysEventService.eventSave(requestCtx, cc.getProjectId(), cc.getDocumentCategory(), cc.getDocumentType(), "FCT", PROPERTY_FCT_CONTRACT_SIGN_SUBMIT, "P2D", evenParams);
    }
}
