package com.hand.hap.activiti.components;

import com.hand.hap.activiti.custom.IActivitiBean;
import com.hand.hap.core.IRequest;
import com.hand.hap.core.impl.RequestHelper;
import hls.core.csh.dto.*;
import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.csh.exception.ChangeLimitException;
import hls.core.csh.exception.WriteOffTypeNullException;
import hls.core.csh.mapper.HlsCusCshTransactionMapper;
import hls.core.csh.mapper.HlsCusTransactionRelationMapper;
import hls.core.csh.service.CshTransactionService;
import hls.core.csh.service.CshWriteOffService;
import hls.core.gld.components.AbstractJeTrxService;
import hls.core.gld.components.JeTrxCommonService;
import hls.core.gld.dto.HlsCusGldLonContractTrx;
import hls.core.gld.service.IGldLonContractTrxService;
import hls.core.hls.components.CalculateUtil;
import hls.core.hls.components.DocumentChangeCommon;
import hls.core.hls.service.IHlsCreditLineTrxService;
import hls.core.lon.dto.HlsCusLonContract;
import hls.core.lon.dto.HlsCusLonContractWithdraw;
import hls.core.lon.mapper.HlsCusLonContractMapper;
import hls.core.lon.mapper.HlsCusLonContractWithdrawMapper;
import hls.core.lon.service.ILonContractWithdrawService;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.JavaDelegate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * Created by fjm on 2017/11/20.
 */
@Component
public class LonWithdrawServiceTask implements JavaDelegate, IActivitiBean {
    public static final String APPROVED = "APPROVED";
    public static final String REJECT = "REJECT";
    public static final String WRITE_OFF_FLAG_FULL = "FULL";//现金流完全核销
    public static final String WRITE_OFF_FLAG_PARTIAL = "PARTIAL";//现金流完全核销
    public static final String LON_RECEIPT_CREDIT = "LON_RECEIPT_CREDIT";
    public static final String REVERSED_FLAG_N = "N";
    public static final String CASH = "CASH";
    public static final String CNY = "CNY";
    public static final String LON_CONTRACT_WITHDRAW = "LON_CONTRACT_WITHDRAW";//融资提款
    public static final String LON_CONTRACT = "LON_CONTRACT";//融资事件的documentCategory
    public static final String NEW = "NEW";
    public static final String LOAN_INCEPT = "LOAN_INCEPT";
    @Autowired
    private CshWriteOffService writeOffService;
    @Autowired
    private ILonContractWithdrawService lonContractWithdrawService;
    @Autowired
    private HlsCusLonContractMapper lonContractMapper;
    @Autowired
    private CshTransactionService transactionService;
    @Autowired
    private HlsCusLonContractWithdrawMapper lonContractWithdrawMapper;
    @Autowired
    private HlsCusTransactionRelationMapper transactionRelationMapper;
    @Autowired
    private HlsCusCshTransactionMapper transactionMapper;
    @Autowired
    private IHlsCreditLineTrxService hlsCreditLineTrxService;
    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private JeTrxCommonService commonService;
    @Autowired
    private IGldLonContractTrxService gldLonContractTrxService;

    @Override
    public void execute(DelegateExecution delegateExecution) {
        IRequest requestCtx = RequestHelper.getCurrentRequest();
        String result = (String) delegateExecution.getVariable("approveResult");
        String processInstanceBusinessKey = delegateExecution.getProcessInstanceBusinessKey();
        long withdrawId = Long.parseLong(processInstanceBusinessKey);
        HlsCusLonContractWithdraw lonContractWithdraw = new HlsCusLonContractWithdraw();
        lonContractWithdraw.setWithdrawId(withdrawId);
        List<HlsCusLonContractWithdraw> select = lonContractWithdrawMapper.select(lonContractWithdraw);
        HlsCusLonContractWithdraw withdraw = select.stream().findAny().orElseGet(HlsCusLonContractWithdraw::new);
        Long contractId = withdraw.getContractId();
        HlsCusLonContract lonContract = lonContractMapper.selectByPrimaryKey(contractId);
        Double dueAmount = withdraw.getDueAmount();
        String currency = CNY;
        String paymentMethod = CASH;
        Long bankAccountId = lonContract.getWithdrawBankAccountId();
        Long creditBpId = lonContract.getCreditBpId();
        Date transactionDate = new Date();
        HlsCusCshTransaction cshTransaction = new HlsCusCshTransaction();
        cshTransaction.setTransactionAmount(dueAmount);
        cshTransaction.setCurrencyCode(currency);
        cshTransaction.setBankAccountId(bankAccountId);
        cshTransaction.setPaymentMethod(paymentMethod);
        cshTransaction.setBpId(creditBpId);
        cshTransaction.setTransactionDate(transactionDate);
        cshTransaction.setLonContractId(contractId);
        cshTransaction = transactionService.updateCshTransaction(requestCtx.getCompanyId(), cshTransaction, requestCtx);

        List<HlsCusCshWriteOff> list = new ArrayList<>();
        HlsCusCshWriteOff writeOffObj = getWriteOffObj(withdraw);
        writeOffObj.setTransactionId(cshTransaction.getTransactionId());
        writeOffObj.setCshTransactionId(cshTransaction.getTransactionId());
        writeOffObj.setWriteOffFlag(WRITE_OFF_FLAG_FULL);
        writeOffObj.setCshWriteOffAmount(dueAmount);
        writeOffObj.setWriteOffDueAmount(dueAmount);
        list.add(writeOffObj);

        sysEventService.createEvent(requestCtx, withdrawId, LON_CONTRACT, "", null, SysEventCodeUtil.PROPERTY_LON_CONTRACT_WITHDRAW_CREATE);

        try {
            writeOffService.updateWriteOff(requestCtx, list, requestCtx.getCompanyId());
        } catch (BeyondAmountLimitException e) {
            e.printStackTrace();
        } catch (WriteOffTypeNullException e) {
            e.printStackTrace();
        } catch (ChangeLimitException e) {
            e.printStackTrace();
        }

        if ("APPROVED".equalsIgnoreCase(result)) {
            withdraw.setWithdrawStatus(APPROVED);
            //插入融资合同成本事物
            HlsCusGldLonContractTrx gldLonContractTrx = new HlsCusGldLonContractTrx();
            gldLonContractTrx.setCompanyId(requestCtx.getCompanyId());
            gldLonContractTrx.setContractId(withdraw.getContractId());
            gldLonContractTrx.setSourceType(LOAN_INCEPT);
            gldLonContractTrx.setSourceId(withdrawId);
            gldLonContractTrx.setStatus(NEW);
            gldLonContractTrxService.insertSelective(requestCtx,gldLonContractTrx);
        } else {
            withdraw.setWithdrawStatus(REJECT);
        }
        withdraw.setWriteOffAmount(dueAmount);
        lonContractWithdrawService.updateByPrimaryKeySelective(requestCtx, withdraw);

//        排除核销完成的提款记录
//        HlsCusTransactionRelation hlsCusTransactionRelation = new HlsCusTransactionRelation();
//        hlsCusTransactionRelation.setRelationDocId(withdrawId);
//        hlsCusTransactionRelation.setRelationDocCategory(DocumentChangeCommon.getTableName(HlsCusLonContractWithdraw.class));
//        List<HlsCusTransactionRelation> transactionRelations = transactionRelationMapper.select(hlsCusTransactionRelation);//找出关联的现金事物
//        List<HlsCusCshTransaction> hlsCusCshTransactions = new ArrayList<>();//现金事物集合
//        transactionRelations.forEach(item -> {
//            HlsCusCshTransaction hlsCusCshTransaction = transactionMapper.selectByPrimaryKey(item.getTransactionId());
//            hlsCusCshTransactions.add(hlsCusCshTransaction);
//        });
//
//        List<Map> params = new ArrayList<>();//记录现金事物和对应的剩余金额
//        hlsCusCshTransactions.forEach(item -> {
//            Map param = new HashMap();
//            param.put("leftAmount", CalculateUtil.sub(item.getTransactionAmount(), item.getWriteOffAmount()));
//            param.put("transactionId", item.getTransactionId());
//            params.add(param);
//        });
//        List<HlsCusLonContractWithdraw> lonContractWithdraws = select.stream().filter(item -> !WRITE_OFF_FLAG_FULL.equalsIgnoreCase(item.getWriteOffFlag())).collect(Collectors.toList());
//        try {
//            writeOffService.updateWriteOff(requestCtx, autoGroupWriteOff(lonContractWithdraws, params), requestCtx.getCompanyId());
//        } catch (BeyondAmountLimitException e) {
//            e.printStackTrace();
//        } catch (WriteOffTypeNullException e) {
//            e.printStackTrace();
//        } catch (ChangeLimitException e) {
//            e.printStackTrace();
//        }


    }

    /**
     * 将提款记录分组为核销对象
     *
     * @param lonContractWithdraws
     * @param params
     * @return
     */
    private List<HlsCusCshWriteOff> autoGroupWriteOff(List<HlsCusLonContractWithdraw> lonContractWithdraws, List<Map> params) {
        List<HlsCusCshWriteOff> result = new ArrayList<>();
        params.forEach(item -> {
            lonContractWithdraws.forEach(pro -> {
                Double leftAmount = (Double) item.get("leftAmount");//当前现金事物所剩未被核销的金额
                Long transactionId = (Long) item.get("transactionId");//现金事物ID
                Double writeOffAmount = CalculateUtil.sub(pro.getDueAmount(), pro.getWriteOffAmount());
                if (!WRITE_OFF_FLAG_FULL.equalsIgnoreCase(pro.getWriteOffFlag()) && leftAmount.compareTo(0D) > 0) {
                    if (leftAmount.compareTo(writeOffAmount) == 1) {//当现金事物上的钱足够核销当前现金流,且还有结余时
                        HlsCusCshWriteOff writeOffObj = getWriteOffObj(pro);
                        writeOffObj.setTransactionId(transactionId);
                        writeOffObj.setCshTransactionId(transactionId);
                        writeOffObj.setWriteOffDueAmount(writeOffAmount);
                        writeOffObj.setCshWriteOffAmount(writeOffAmount);
                        item.put("leftAmount", CalculateUtil.sub(leftAmount, writeOffAmount));
                        pro.setWriteOffFlag(WRITE_OFF_FLAG_FULL);
                        pro.setWriteOffAmount(CalculateUtil.add(pro.getWriteOffAmount(), writeOffAmount));
                        result.add(writeOffObj);
                    } else if (leftAmount.compareTo(writeOffAmount) == -1) {//当现金事物上的钱不够核销时
                        pro.setWriteOffAmount(CalculateUtil.add(pro.getWriteOffAmount(), leftAmount));//当现金事物上余额不足是,应该把当前所剩的钱既为该核销记录的应收金额
                        HlsCusCshWriteOff writeOffObj = getWriteOffObj(pro);
                        writeOffObj.setTransactionId(transactionId);
                        writeOffObj.setCshTransactionId(transactionId);
                        writeOffObj.setWriteOffDueAmount(writeOffAmount);
                        writeOffObj.setCshWriteOffAmount(writeOffAmount);
                        item.put("leftAmount", 0D);
                        pro.setWriteOffFlag(WRITE_OFF_FLAG_PARTIAL);
                        pro.setWriteOffAmount(CalculateUtil.add(pro.getWriteOffAmount(), writeOffAmount));
                        result.add(writeOffObj);
                    }
                }
            });
        });
        return result;
    }

    /**
     * 获取核销需要的数据
     *
     * @param item
     * @return
     */
    private static HlsCusCshWriteOff getWriteOffObj(HlsCusLonContractWithdraw item) {
        HlsCusCshWriteOff writeOff = new HlsCusCshWriteOff();
        writeOff.setWriteOffType(LON_RECEIPT_CREDIT);
        writeOff.setWriteOffDate(new Date());
        writeOff.setContractId(item.getContractId());
        writeOff.setCfItem(item.getCfItem());
        writeOff.setCfType(item.getCfType());
        writeOff.setWithdrawId(item.getWithdrawId());
        writeOff.setWriteOffDocCategory(DocumentChangeCommon.getTableName(HlsCusLonContract.class));
        writeOff.setReversedFlag(REVERSED_FLAG_N);
        return writeOff;
    }
}
