package com.hand.hap.activiti.components;

import com.hand.hap.activiti.custom.IActivitiBean;
import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import hls.core.csh.dto.HlsCusCshPaymentReqHd;
import hls.core.csh.service.ICshPaymentReqHdService;
import hls.core.hls.components.DocumentChangeCommon;
import hls.core.hls.dto.HlsCusHlsCreditLineTrx;
import hls.core.hls.service.IHlsCreditLineTrxService;
import hls.core.lon.dto.HlsCusLonContract;
import hls.core.lon.dto.HlsCusLonContractRepayment;
import hls.core.lon.dto.HlsCusLonContractWithdraw;
import hls.core.lon.mapper.HlsCusLonContractMapper;
import hls.core.lon.mapper.HlsCusLonContractRepaymentMapper;
import hls.core.sys.dto.SysUser;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.service.SysUserService;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.JavaDelegate;
import org.apache.bcel.verifier.statics.LONG_Upper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * Created by wangyan on 2017/11/16.
 */
@Component
public class RepaymentServiceTask implements JavaDelegate, IActivitiBean {
    @Autowired
    private ApplicationContext applicationContext;
    @Autowired
    private ICshPaymentReqHdService paymentReqHdService;
    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private SysUserService userService;
    @Autowired
    private HlsCusLonContractMapper lonContractMapper;
    @Autowired
    private HlsCusLonContractRepaymentMapper repaymentMapper;
    @Autowired
    private IHlsCreditLineTrxService hlsCreditLineTrxService;
    @Autowired
    private DatabaseLockProvider databaseLockProvider;
    public static final String LON_CONTRACT_REPAYMENT = "LON_CONTRACT_REPAYMENT";//融资还款

    public RepaymentServiceTask(){}

    @Override
    public void execute(DelegateExecution delegateExecution) {
        String flag = null;
        //获取申请头id
        Long paymentReqHdId = Long.parseLong(delegateExecution.getProcessInstanceBusinessKey());
        IRequest requestContext = (IRequest)delegateExecution.getVariable("iRequest");
        String result = (String)delegateExecution.getVariable("approveResult");
        String employeeCode = (String)delegateExecution.getVariable("startUserName");
        SysUser sysUser = userService.queryUserByCode(employeeCode);
        //获取融资合同id
        Long contractId = (Long)delegateExecution.getVariable("contractId");
        //获取授信合同id
        Long creditLineId = lonContractMapper.selectByPrimaryKey(contractId).getCreditLineId();
        //获取申请金额列表
        String amountListString = (String)delegateExecution.getVariable("amountList");
        String []  amountList =amountListString.split("-");
        //获取还款id列表
        String  repaymentIds = (String)delegateExecution.getVariable("repaymentIds");
        String [] idsArray = repaymentIds.split("-");
        if(amountList.length!=idsArray.length){
            return ;
        }
        if("APPROVED".equalsIgnoreCase(result))
        {
            flag ="APPROVED";
            for(int i=0;i<idsArray.length;i++){
                //插入额度事物表
                HlsCusHlsCreditLineTrx hlsCreditLineTrx = new HlsCusHlsCreditLineTrx();
                hlsCreditLineTrx.setCompanyId(requestContext.getCompanyId());
                hlsCreditLineTrx.setTrxCode(LON_CONTRACT_REPAYMENT);
                hlsCreditLineTrx.setTrxAmount(Double.valueOf(amountList[i]));
                hlsCreditLineTrx.setSourceDocumentId(Long.parseLong(idsArray[i]));
                hlsCreditLineTrx.setSourceDocumentCategory(DocumentChangeCommon.getTableName(HlsCusLonContractRepayment.class));
                hlsCreditLineTrx.setCreditLineId(creditLineId);
                //获取due_date
                HlsCusLonContractRepayment repayment = repaymentMapper.selectByPrimaryKey(Long.parseLong(idsArray[i]));
                Date dueDate = repayment.getDueDate();
                hlsCreditLineTrx.setTrxDate(dueDate);

                //更新额度主数据上对应的已占用额度(只根据本金更新)
                if(repayment.getCfItem().compareTo(301L)==0){
                    hlsCreditLineTrx = hlsCreditLineTrxService.insertSelective(requestContext, hlsCreditLineTrx);
                    hlsCreditLineTrxService.calcCreditMethod(requestContext, hlsCreditLineTrx.getTrxId());
                }
            }



        }else if("CANCEL".equalsIgnoreCase(result))
        {
            flag ="CANCEL";
        }else
        {
            flag ="REJECTED";
        }

        HlsCusCshPaymentReqHd hd = new HlsCusCshPaymentReqHd();
        hd.setPayment_req_id(paymentReqHdId);
        databaseLockProvider.lock(hd);
        hd.setPayment_req_status(flag);
        hd.setProcessInstanceId(Long.parseLong(delegateExecution.getProcessInstanceId()));
        paymentReqHdService.updateByPrimaryKeySelective(requestContext,hd);

    }

}
