package com.hand.hap.activiti.controllers;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.hand.hap.activiti.components.ApprovalRule;
import com.hand.hap.activiti.core.IActivitiConstants;
import com.hand.hap.activiti.custom.process.CustomHistoricProcessInstanceQueryRequest;
import com.hand.hap.activiti.custom.task.CustomTaskQueryRequest;
import com.hand.hap.activiti.dto.*;
import com.hand.hap.activiti.exception.TaskActionException;
import com.hand.hap.activiti.exception.WflSecurityException;
import com.hand.hap.activiti.exception.dto.ActiviException;
import com.hand.hap.activiti.service.ActHiTaskinstService;
import com.hand.hap.activiti.service.IActiviCancelService;
import com.hand.hap.activiti.service.IActivitiEntityService;
import com.hand.hap.activiti.service.IActivitiService;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.hls.dto.HlsSystemNotice;
import hls.core.hls.dto.HlsSystemNoticeOwner;
import hls.core.hls.service.HlsSystemNoticeOwnerService;
import hls.core.hls.service.HlsSystemNoticeService;
import hls.core.sys.dto.SysUser;
import hls.core.sys.mapper.SysUserMapper;
import org.activiti.bpmn.converter.BpmnXMLConverter;
import org.activiti.bpmn.model.BpmnModel;
import org.activiti.bpmn.model.FlowElement;
import org.activiti.bpmn.model.SequenceFlow;
import org.activiti.editor.language.json.converter.BpmnJsonConverter;
import org.activiti.engine.*;
import org.activiti.engine.history.HistoricActivityInstance;
import org.activiti.engine.history.HistoricProcessInstance;
import org.activiti.engine.history.HistoricVariableInstance;
import org.activiti.engine.impl.util.io.InputStreamSource;
import org.activiti.engine.repository.Model;
import org.activiti.engine.repository.ProcessDefinition;
import org.activiti.engine.runtime.ProcessInstance;
import org.activiti.image.ProcessDiagramGenerator;
import org.activiti.rest.common.api.DataResponse;
import org.activiti.rest.service.api.RestResponseFactory;
import org.activiti.rest.service.api.history.HistoricTaskInstanceQueryRequest;
import org.activiti.rest.service.api.repository.ModelRequest;
import org.activiti.rest.service.api.repository.ModelResponse;
import org.activiti.rest.service.api.runtime.process.ProcessInstanceCreateRequest;
import org.activiti.rest.service.api.runtime.process.ProcessInstanceResponse;
import org.activiti.rest.service.api.runtime.task.TaskQueryRequest;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;
/**
 * @author shengyang.zhou@hand-china.com
 */
@Controller
@RequestMapping(value = "/wfl")
public class ActivitiController extends BaseController {

    @Autowired
    private RepositoryService repositoryService;

    @Autowired
    private RuntimeService runtimeService;

    @Autowired
    private RestResponseFactory restResponseFactory;

    @Autowired
    private IActivitiEntityService customEntityService;

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private IActivitiService activitiService;

    @Autowired
    private ActHiTaskinstService actHiTaskinstService;

    @Autowired
    protected ProcessEngineConfiguration processEngineConfiguration;

    @Autowired
    private HistoryService historyService;
    @Autowired
    private SysUserMapper userMapper;
    @Autowired
    private HlsSystemNoticeService hlsSystemNoticeService;
    @Autowired
    private HlsSystemNoticeOwnerService hlsSystemNoticeOwnerService;
    @Autowired
    private IActiviCancelService activiCancelService;

    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private ApprovalRule approvalRule;

    /**
     * 创建流程,使用当前登录用户.
     *
     * @param request
     * @param httpRequest
     * @param response
     * @return
     */
    @RequestMapping(value = "/runtime/process-instances", method = RequestMethod.POST, produces = "application/json")
    public ProcessInstanceResponse createProcessInstance(@RequestBody ProcessInstanceCreateRequest request,
                                                         HttpServletRequest httpRequest, HttpServletResponse response) {
        IRequest iRequest = createRequestContext(httpRequest);
        String employeeCode = httpRequest.getParameter("userId");
        if (StringUtils.isNotEmpty(employeeCode) && "ADMIN".equalsIgnoreCase(iRequest.getEmployeeCode())) {
            // allow ADMIN to simulate other user
            // FOR TEST ONLY
            iRequest.setEmployeeCode(employeeCode);
        }
        return activitiService.startProcess(iRequest, request);
    }

    /*添加抄送信息*/
    @RequestMapping(value = "/add/copyMessage", method = RequestMethod.GET)
    public ResponseData addCopyMessage(String commentId,String message,
                                                         HttpServletRequest httpRequest, HttpServletResponse response) {
        IRequest iRequest = createRequestContext(httpRequest);
        ResponseData data = new ResponseData();
        try {
            activitiService.addComment(commentId, message,iRequest);
        }catch (Exception e){
            data.setMessage("添加抄送信息失败！");
        }
        data.setMessage("添加抄送信息成功!");
        return data;
    }

    //通过任务id查询form url
    @RequestMapping(value = "/taskId/query", method = RequestMethod.GET)
    public ResponseData addCopyMessage(String taskId,
                                       HttpServletRequest httpRequest, HttpServletResponse response) {
        IRequest iRequest = createRequestContext(httpRequest);
        ResponseData data = new ResponseData();
        try {
            String formKey = actHiTaskinstService.queryFormKey(taskId);
            data.setMessage(formKey);
        }catch (Exception e){
            data.setMessage("error");
        }
        return data;
    }

    @RequestMapping(value = "/definition/user-tasks", method = RequestMethod.GET)
    @ResponseBody
    public List<ActivitiNode> getAvailableUserTask(HttpServletRequest request, String processDefinitionId) {
        IRequest iRequest = createRequestContext(request);
        return activitiService.getProcessNodes(iRequest, processDefinitionId);
    }

    /**
     * 待办事项,个人（强制根据员工号过滤）
     *
     * @param request
     * @param requestParams
     * @param httpRequest
     * @return
     */
    @RequestMapping(value = "/query/tasks", method = RequestMethod.POST, produces = "application/json")
    public DataResponse getQueryResult(@RequestBody CustomTaskQueryRequest request,
                                       @RequestParam Map<String, String> requestParams, HttpServletRequest httpRequest) {
        IRequest iRequest = createRequestContext(httpRequest);
        SysUser u = new SysUser();
        u.setUserId(iRequest.getUserId());
        SysUser user = userMapper.selectOne(u);
        //request.setCandidateOrAssigned(iRequest.getEmployeeCode());
        request.setCandidateOrAssigned(user.getUserName());
        DataResponse dataResponse = activitiService.queryTaskList(iRequest, request, requestParams);
        return dataResponse;
    }

    /**
     * 待办事项，管理员用(可以任意查询)
     *
     * @param request
     * @param requestParams
     * @param httpRequest
     * @return
     */
    @RequestMapping(value = "/query/tasks/admin", method = RequestMethod.POST, produces = "application/json")
    public DataResponse taskListAdmin(@RequestBody TaskQueryRequest request,
                                      @RequestParam Map<String, String> requestParams, HttpServletRequest httpRequest) {

        IRequest iRequest = createRequestContext(httpRequest);
        if(request.getCandidateOrAssigned() != null){
            request.setName(request.getCandidateOrAssigned());
        }
        DataResponse dataResponse = activitiService.queryTaskList(iRequest, request, requestParams);
        return dataResponse;
    }

    /**
     * 审批记录
     *
     * @param queryRequest
     * @param allRequestParams
     * @param request
     * @return
     */
    @RequestMapping(value = "/query/historic-task-instances")
    public DataResponse queryProcessInstances(HistoricTaskInstanceQueryRequest queryRequest,
                                              @RequestParam Map<String, String> allRequestParams, HttpServletRequest request) {

        IRequest iRequest = createRequestContext(request);
        return activitiService.queryHistoricTaskInstances(iRequest, queryRequest, allRequestParams);
    }

    /**
     * 我参与的流程
     *
     * @param historicProcessInstanceQueryRequest
     * @param requestParams
     * @param httpRequest
     * @return dataResponse
     */
    @RequestMapping(value = "/query/process-instances/my")
    public DataResponse queryProcessInstances(@RequestBody CustomHistoricProcessInstanceQueryRequest historicProcessInstanceQueryRequest,
                                              @RequestParam Map<String, String> requestParams, HttpServletRequest httpRequest) throws WflSecurityException {

        IRequest iRequest = createRequestContext(httpRequest);
        return activitiService.queryProcessInstances(iRequest, historicProcessInstanceQueryRequest, requestParams, true);
    }


    /**
     * 流程监控查询.
     *
     * @param historicProcessInstanceQueryRequest
     * @param requestParams
     * @param httpRequest
     * @return dataResponse
     */
    @RequestMapping(value = "/query/process-instances/monitor")
    public DataResponse queryAllProcessInstances(@RequestBody CustomHistoricProcessInstanceQueryRequest historicProcessInstanceQueryRequest,
                                                 @RequestParam Map<String, String> requestParams, HttpServletRequest httpRequest) {

        IRequest iRequest = createRequestContext(httpRequest);
        return activitiService.queryProcessInstances(iRequest, historicProcessInstanceQueryRequest, requestParams, false);
    }

    /**
     * 完成,转交...任务
     *
     * @param taskId
     * @param actionRequest
     * @param request
     * @param response
     */
    @RequestMapping(value = "/runtime/tasks/{taskId}", method = RequestMethod.POST)
    @ResponseStatus(value = HttpStatus.OK)
    public void executeTaskAction(@PathVariable String taskId, @RequestBody TaskActionRequestExt actionRequest,
                                  HttpServletRequest request, HttpServletResponse response) throws TaskActionException {
        IRequest iRequest = createRequestContext(request);
        activitiService.executeTaskAction(iRequest, taskId, actionRequest,false);
    }

    @RequestMapping(value = "/runtime/admin/tasks/{taskId}", method = RequestMethod.POST)
    @ResponseStatus(value = HttpStatus.OK)
    public void executeTaskActionAdmin(@PathVariable String taskId, @RequestBody TaskActionRequestExt actionRequest,
                                       HttpServletRequest request, HttpServletResponse response) throws TaskActionException {
        IRequest iRequest = createRequestContext(request);
        activitiService.executeTaskAction(iRequest, taskId, actionRequest,true);
    }

    @RequestMapping("/runtime/admin/tasks/{taskId}/details")
    @ResponseBody
    public TaskResponseExt taskDetailsAdmin(HttpServletRequest request, @PathVariable String taskId)
            throws WflSecurityException {

        IRequest iRequest = createRequestContext(request);
        return activitiService.getTaskDetails(iRequest, taskId,true);
    }

    @RequestMapping(value = "/runtime/admin/tasks/batch-delegate", method = RequestMethod.POST)
    @ResponseStatus(value = HttpStatus.OK)
    public void executeBatchDelegate(@RequestBody List<TaskActionRequestExt> actionRequests,
                                     HttpServletRequest request, HttpServletResponse response) throws TaskActionException {
        IRequest iRequest = createRequestContext(request);
        for(TaskActionRequestExt actionRequest:actionRequests){
            actionRequest.setComment("");
            activitiService.executeTaskAction(iRequest, actionRequest.getCurrentTaskId(), actionRequest,true);
        }
    }

    /**
     * 新建模型(editor)
     *
     * @param modelRequest
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/repository/models", method = RequestMethod.POST, produces = "application/json")
    public ModelResponse createModel(@RequestBody ModelRequest modelRequest, HttpServletRequest request,
                                     HttpServletResponse response) {
        Model model = repositoryService.newModel();
        model.setCategory(modelRequest.getCategory());
        model.setDeploymentId(modelRequest.getDeploymentId());
        model.setKey(modelRequest.getKey());
        model.setMetaInfo(modelRequest.getMetaInfo());
        model.setName(modelRequest.getName());
        model.setVersion(modelRequest.getVersion());
        model.setTenantId(modelRequest.getTenantId());

        repositoryService.saveModel(model);
        response.setStatus(HttpStatus.CREATED.value());

        HashMap<String, Object> content = new HashMap<>();
        content.put("resourceId", model.getId());

        HashMap<String, String> properties = new HashMap<>();
        properties.put("process_id", modelRequest.getKey());
        properties.put("name", modelRequest.getName());
        properties.put("process_namespace", modelRequest.getCategory());
        content.put("properties", properties);

        HashMap<String, String> stencilset = new HashMap<>();
        stencilset.put("namespace", "http://b3mn.org/stencilset/bpmn2.0#");
        content.put("stencilset", stencilset);

        try {
            repositoryService.addModelEditorSource(model.getId(), objectMapper.writeValueAsBytes(content));
        } catch (JsonProcessingException e) {
            logger.error(e.getMessage(), e);
        }

        return restResponseFactory.createModelResponse(model);

    }

    @RequestMapping("/repository/model/node")
    public ResponseData getUserTaskFromModelSource(HttpServletRequest request, String modelId) {
        IRequest iRequest = createRequestContext(request);
        List<?> list = activitiService.getUserTaskFromModelSource(iRequest, modelId);
        return new ResponseData(list);
    }

    /**
     * 部署流程
     *
     * @param modelId
     * @throws IOException
     */
    @RequestMapping("/repository/model/{modelId}/deploy")
    @ResponseBody
    public ResponseData modelDeployment(@PathVariable String modelId) {
        Model model = null;
        try {
            model = activitiService.deployModel(modelId);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            ResponseData rd = new ResponseData(false);
            rd.setMessage(e.getMessage());
            return rd;
        }
        return new ResponseData(Arrays.asList(model));
    }

    @RequestMapping(value = "/repository/model/{modelId}/export", produces = "text/xml")
    public ResponseEntity<byte[]> modelExport(@PathVariable String modelId,
                                              @RequestParam(defaultValue = "") String type) throws IOException {
        Model model = repositoryService.getModel(modelId);
        byte[] modelData = repositoryService.getModelEditorSource(modelId);
        JsonNode jsonNode = objectMapper.readTree(modelData);
        BpmnModel bpmnModel = new BpmnJsonConverter().convertToBpmnModel(jsonNode);
        byte[] xmlBytes = new BpmnXMLConverter().convertToXML(bpmnModel, "UTF-8");
        HttpHeaders responseHeaders = new HttpHeaders();
        String id = model.getId();
        if (bpmnModel.getMainProcess() != null) {
            id = bpmnModel.getMainProcess().getId();
        }
        if ("bpmn20".equalsIgnoreCase(type)) {
            responseHeaders.set("Content-Disposition", "attachment;filename=" + id + ".bpmn20.xml");
            responseHeaders.set("Content-Type", "application/octet-stream");
        } else {
            responseHeaders.set("Content-Type", "text/xml;charset=utf8");
        }
        try {
            return new ResponseEntity<>(xmlBytes, responseHeaders, HttpStatus.OK);
        } catch (Exception e) {
            throw new ActivitiIllegalArgumentException("Error exporting diagram", e);
        }
    }

    @RequestMapping("/repository/deploy/{deployId}/export")
    public ResponseEntity<byte[]> deployExport(@PathVariable String deployId) throws IOException {
        BpmnModel bpmnModel = repositoryService.getBpmnModel(deployId);
        byte[] xmlBytes = new BpmnXMLConverter().convertToXML(bpmnModel, "UTF-8");
        HttpHeaders responseHeaders = new HttpHeaders();
        responseHeaders.set("Content-Type", "text/xml;charset=utf8");
        try {
            return new ResponseEntity<>(xmlBytes, responseHeaders, HttpStatus.OK);
        } catch (Exception e) {
            throw new ActivitiIllegalArgumentException("Error exporting diagram", e);
        }
    }

    @RequestMapping(value = "/repository/model/import", produces = "text/html;charset=UTF-8")
    public String importModel(HttpServletRequest request) throws FileUploadException, IOException {
        if (!ServletFileUpload.isMultipartContent(request)) {
            return "<script>alert('NOT a Multipart Request')</script>";
        }
        InputStream fileInputStream = null;
        String name = null;
        String key = null;
        String category = null;
        if (request instanceof MultipartHttpServletRequest) {
            MultipartFile file = ((MultipartHttpServletRequest) request).getFile("file");
            if (file != null) {
                fileInputStream = file.getInputStream();
            }
            name = request.getParameter("name");
            key = request.getParameter("key");
            category = request.getParameter("category");
        } else {
            ServletFileUpload upload = new ServletFileUpload(new DiskFileItemFactory());
            List<FileItem> items = upload.parseRequest(request);
            FileItem item = null;
            Map<String, String> parameters = new HashMap<>();
            for (FileItem fi : items) {
                if (!fi.isFormField()) {
                    item = fi;
                } else {
                    parameters.put(fi.getFieldName(), fi.getString("UTF-8"));
                }
            }
            if (item != null && StringUtils.isNotEmpty(item.getName())) {
                fileInputStream = item.getInputStream();
                name = parameters.get("name");
                key = parameters.get("key");
                category = parameters.get("category");
            }
        }
        if (fileInputStream == null) {
            return "<script>alert('File Content is Null')</script>";
        }
        try (InputStream inputStream = fileInputStream) {
            InputStreamSource source = new InputStreamSource(fileInputStream);
            BpmnModel bpmnModel = new BpmnXMLConverter().convertToBpmnModel(source, false, false, "UTF-8");
            bpmnModel.getMainProcess().setId(StringUtils.defaultIfEmpty(key, bpmnModel.getMainProcess().getId()));
            bpmnModel.getMainProcess().setName(StringUtils.defaultIfEmpty(name, bpmnModel.getMainProcess().getName()));

            Model model = repositoryService.newModel();
            model.setCategory(StringUtils.defaultIfEmpty(category, "default"));
            model.setDeploymentId(null);
            model.setKey(bpmnModel.getMainProcess().getId());
            model.setName(bpmnModel.getMainProcess().getName());

            String metaInfo = model.getMetaInfo();
            if (StringUtils.isEmpty(metaInfo)) {
                HashMap<String, String> map = new HashMap<>();
                map.put("name", model.getName());
                map.put("version", String.valueOf(model.getVersion()));
                metaInfo = objectMapper.writeValueAsString(map);
            }
            model.setMetaInfo(metaInfo);

            repositoryService.saveModel(model);

            ObjectNode content = new BpmnJsonConverter().convertToJson(bpmnModel);
            content = approvalRule.processCustomProperties(content);

            repositoryService.addModelEditorSource(model.getId(), objectMapper.writeValueAsBytes(content));
            restResponseFactory.createModelResponse(model);
            return "<script>window.parent.onImportComplete(true)</script>";
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return "<script>window.parent.onImportComplete(false,'请上传符合 BPMN 规范的文件')</script>";
        }
    }

    @RequestMapping("/runtime/tasks/{taskId}/details")
    @ResponseBody
    public TaskResponseExt taskDetails(HttpServletRequest request, @PathVariable String taskId)
            throws WflSecurityException {

        IRequest iRequest = createRequestContext(request);
        return activitiService.getTaskDetails(iRequest, taskId);
    }



    @RequestMapping(value = "/runtime/process-instances/{processInstanceId}/diagram", method = RequestMethod.GET)
    public ResponseEntity<byte[]> getProcessInstanceDiagram(@PathVariable String processInstanceId,
                                                            HttpServletResponse response) {

        //先判断是否在历史进行的流程中
        boolean isActivity =  runtimeService.createProcessInstanceQuery()
                .processInstanceId(processInstanceId).singleResult() == null?false:true;
        //查询流程历史
        HistoricProcessInstance processInstance = historyService.createHistoricProcessInstanceQuery().processInstanceId(processInstanceId).singleResult();

        ProcessDefinition pde = repositoryService.getProcessDefinition(processInstance.getProcessDefinitionId());

        if (pde != null && pde.hasGraphicalNotation()) {
            BpmnModel bpmnModel = repositoryService.getBpmnModel(pde.getId());
            ProcessDiagramGenerator diagramGenerator = processEngineConfiguration.getProcessDiagramGenerator();

            List<HistoricActivityInstance> finished = historyService.createHistoricActivityInstanceQuery()
                    .processInstanceId(processInstanceId).finished().list();
            List<String> ids = new ArrayList<>(finished.stream()
                    .map(e -> e.getActivityId() + IActivitiConstants.HISTORY_SUFFIX).collect(Collectors.toList()));
            if(isActivity) {
                ids.addAll(runtimeService.getActiveActivityIds(processInstance.getId()));
            }

            List<String> flows = new ArrayList<>();
            Collection<FlowElement> flowElement = bpmnModel.getMainProcess().getFlowElements();

            String startId;
            String endId;
            for (int i = 0; i < ids.size() - 1; i++) {
                startId = ids.get(i);
                endId = ids.get(i + 1);

                if (endId.startsWith("eg_auto")) {
                    endId = ids.get(++i + 1);
                }

                if (startId.endsWith(IActivitiConstants.HISTORY_SUFFIX)) {
                    startId = startId.substring(0, startId.indexOf(IActivitiConstants.HISTORY_SUFFIX));
                }
                if (endId.endsWith(IActivitiConstants.HISTORY_SUFFIX)) {
                    endId = endId.substring(0, endId.indexOf(IActivitiConstants.HISTORY_SUFFIX));
                }

                for (FlowElement flowElement1 : flowElement) {
                    if (flowElement1 instanceof SequenceFlow)
                        if (((SequenceFlow) flowElement1).getSourceRef().equals(startId) && ((SequenceFlow) flowElement1).getTargetRef().equals(endId)) {
                            flows.add(flowElement1.getId());
                        }
                }
            }

            InputStream resource = diagramGenerator.generateDiagram(bpmnModel, "svg", ids,
                    flows, processEngineConfiguration.getActivityFontName(),
                    processEngineConfiguration.getLabelFontName(), processEngineConfiguration.getAnnotationFontName(),
                    processEngineConfiguration.getClassLoader(), 1.0);

            HttpHeaders responseHeaders = new HttpHeaders();
            responseHeaders.set("Content-Type", "image/svg+xml");
            try {
                return new ResponseEntity<>(IOUtils.toByteArray(resource), responseHeaders, HttpStatus.OK);
            } catch (Exception e) {
                throw new ActivitiIllegalArgumentException("Error exporting diagram", e);
            }

        } else {
            throw new ActivitiIllegalArgumentException(
                    "Process instance with id '" + processInstance.getId() + "' has no graphical notation defined.");
        }
    }

    @RequestMapping("/history/form/details/{procId}")
    @ResponseBody
    public ResponseData getFromVar(HttpServletRequest request, @PathVariable String procId) {
        List<HistoricVariableInstance> list = historyService.createHistoricVariableInstanceQuery().processInstanceId(procId).list();
//        for (HistoricVariableInstance variable : list) {
//            System.out.println("variable: " + variable.getVariableName() + " = " + variable.getValue());
//        }

        return new ResponseData(list);
    }

    @RequestMapping("/instance/{processInstanceId}/{currentTaskId}")
    @ResponseBody
    public HistoricProcessInstanceResponseExt instanceDetail(HttpServletRequest request, @PathVariable String processInstanceId ,@PathVariable String currentTaskId)
            throws WflSecurityException {

        IRequest iRequest = createRequestContext(request);
        return activitiService.getInstanceDetail(iRequest, processInstanceId);
    }

    @RequestMapping("/instance/{processInstanceId}")
    @ResponseBody
    public HistoricProcessInstanceResponseExt instanceDetail(HttpServletRequest request, @PathVariable String processInstanceId)
            throws WflSecurityException {

        IRequest iRequest = createRequestContext(request);
        return activitiService.getInstanceDetail(iRequest, processInstanceId);
    }

    @RequestMapping("/runtime/prc/suspend/{procId}")
    public void suspendProc(HttpServletRequest request, @PathVariable String procId) {
        runtimeService.suspendProcessInstanceById(procId);
    }

    @RequestMapping("/runtime/prc/active/{procId}")
    public void activeProc(HttpServletRequest request, @PathVariable String procId) {
        runtimeService.activateProcessInstanceById(procId);
    }

    @RequestMapping(value = "/runtime/prc/end/{procId}", method = RequestMethod.POST)
    @ResponseBody
    public void endProc(HttpServletRequest request, @PathVariable String procId) {
        runtimeService.deleteProcessInstance(procId, IActivitiConstants.ACT_STOP);

    }

    @RequestMapping(value = "/runtime/prc/back/{procId}", method = RequestMethod.POST)
    @ResponseBody
    public Map backProc(HttpServletRequest request, @PathVariable String procId) throws Exception {
        IRequest iRequest = createRequestContext(request);
        Map result = new HashMap<>();
        result.put("success",true);
        result.put("message","撤回成功");
        if (activitiService.isStartRecall(procId,iRequest.getEmployeeCode())) {
            activiCancelService.cancelWorkFlow(procId,iRequest);
            runtimeService.deleteProcessInstance(procId, IActivitiConstants.ACT_RETRACT);
        }else if (activitiService.isTaskRecall(procId, iRequest.getEmployeeCode())) {
            activiCancelService.cancelWorkFlow(procId,iRequest);
            activitiService.taskRecall(procId, iRequest.getEmployeeCode());
        }else{
            result.put("success",false);
            result.put("message","已发生过审批的流程不允许发起人撤回");
        }
        return result;
    }

    @RequestMapping(value = "/runtime/exception/search", method = RequestMethod.POST)
    @ResponseBody
    public ResponseData searchException(HttpServletRequest request, ActiviException exception, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                        @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pagesize) {
        return new ResponseData(activitiService.queryException(exception, page, pagesize));
    }


    @RequestMapping(value = "/runtime/execute/{procId}", method = RequestMethod.POST)
    @ResponseStatus(value = HttpStatus.OK)
    @ResponseBody
    public ResponseData executeTaskActionByAdmin(@PathVariable String procId, @RequestBody TaskActionRequestExt actionRequest,
                                                 HttpServletRequest request, HttpServletResponse response) throws TaskActionException {
        actionRequest.setComment("no comment");
        IRequest iRequest = createRequestContext(request);
        activitiService.executeTaskByAdmin(iRequest, procId, actionRequest);
        return new ResponseData();
    }

    /**
     * 审批通过给该条消息设为已办
     *
     */
    @RequestMapping(value = "/save/done/tasks")
    @ResponseBody
    public ResponseData setReadFlag(final HlsSystemNotice hsn, final HttpServletRequest request) {
        IRequest iRequest = createRequestContext(request);
        List<HlsSystemNotice> hsnList = hlsSystemNoticeService.queryNoticeByTodoAndDocId(Long.parseLong(hsn.getQueryTermValue()));
        HlsSystemNoticeOwner hlsSystemNoticeOwner = new HlsSystemNoticeOwner();
        List<HlsSystemNoticeOwner> hsnoList = new ArrayList<>();
        if(hsnList.size()>0){
            hlsSystemNoticeOwner.setNotice_id(hsnList.get(0).getNotice_id());
            hlsSystemNoticeOwner.setOwner_user_id(hsnList.get(0).getSource_user_id());
            hlsSystemNoticeOwner.setRead_flag("Y");
            hlsSystemNoticeOwner.setDone_flag("Y");
            List<HlsSystemNoticeOwner> hlsSystemNoticeOwnerList = hlsSystemNoticeOwnerService.queryOwnerNoticeById(hlsSystemNoticeOwner);
            if(hlsSystemNoticeOwnerList.size()>0){
                hlsSystemNoticeOwner.set__status("update");
            }else {
                hlsSystemNoticeOwner.set__status("add");
            }
            hsnoList.add(hlsSystemNoticeOwner);
            hlsSystemNoticeOwnerService.batchUpdate(iRequest,hsnoList);
        }
        return new ResponseData(hsnoList);
    }

    @RequestMapping(value="/runtime/process-instances/{processInstanceId}/forecast")
    public List<ProcessInstanceForecast> getProcessInstanceForecast (HttpServletRequest request,@PathVariable String processInstanceId){
        IRequest iRequest = createRequestContext(request);
        return activitiService.processInstanceForecast(iRequest,processInstanceId);
    }

}
