package com.hand.hap.activiti.demo.components;

import com.hand.hap.activiti.custom.JumpActivityCmd;
import com.hand.hap.activiti.dto.ActivitiNode;
import org.activiti.bpmn.model.BpmnModel;
import org.activiti.bpmn.model.FlowElement;
import org.activiti.bpmn.model.UserTask;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.TaskService;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.JavaDelegate;
import org.activiti.engine.impl.cfg.ProcessEngineConfigurationImpl;
import org.activiti.engine.task.Task;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

import com.hand.hap.activiti.custom.IActivitiBean;

import java.util.Collection;
import java.util.List;

/**
 * 实现接口JavaDelegate ,IActivitiBean. <br>
 * 定义在 spring context 中,可以 delegateExpression 中用
 * 
 * @author shengyang.zhou@hand-china.com
 */
@Component // 自动扫描注册(components)
public class DemoServiceTaskDelegate implements JavaDelegate, IActivitiBean {

    private Logger logger = LoggerFactory.getLogger(DemoServiceTaskDelegate.class);

    @Autowired
    private ApplicationContext applicationContext;// 可以注入 bean
    @Autowired
    private ProcessEngineConfigurationImpl processEngineConfiguration;
    @Autowired
    private RepositoryService repositoryService;
    @Autowired
    private RuntimeService runtimeService;
    @Autowired
    private TaskService taskService;

    @Override
    public void execute(DelegateExecution execution) {
        logger.debug("get variable :task1Output=" + execution.getVariable("task1Output"));
        logger.debug(":::" + execution.getCurrentFlowElement());
        execution.setVariable("task2Output", "value2");
        String taskId = execution.getId();
        BpmnModel bpmnModel = repositoryService.getBpmnModel(execution.getProcessDefinitionId());
        Collection<FlowElement> flowElements = bpmnModel.getMainProcess().getFlowElements();
        for (FlowElement element : flowElements) {
                System.out.println("flowelement id:" + element.getId() + "  name:" + element.getName() + "   class:" + element.getClass().toString());
        }
        List<Task> taskList = taskService.createTaskQuery().processDefinitionId(execution.getProcessDefinitionId()).list();
        //JumpActivityCmd cmd = new JumpActivityCmd(taskId, "sid-qYEJkajH-i9nK-4PCR-8CDA-jWqjzOq1mNnp");
        //processEngineConfiguration.getCommandExecutor().execute(cmd);
    }

    /**
     * 该方法随便写的，和接口没关系
     *
     * @param delegateExecution
     */
    public void method1(DelegateExecution delegateExecution, String inputValue) {
        logger.debug("method1 called.");
        logger.debug("当前节点：" + delegateExecution.getCurrentFlowElement());
        logger.debug("参数值：" + inputValue);
    }

    @Override
    public String getBeanName(){
        return "demoServiceTaskDelegate";
    }
}
