package com.hand.hap.activiti.service.impl;

import com.hand.hap.activiti.exception.WorkflowTypeNullException;
import com.hand.hap.activiti.service.IActiviCancelService;
import com.hand.hap.core.IRequest;
import hls.core.wfl.service.IActivitiCommonService;
import org.activiti.engine.HistoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.history.HistoricActivityInstance;
import org.activiti.engine.history.HistoricProcessInstance;
import org.activiti.engine.impl.persistence.entity.VariableInstance;
import org.activiti.engine.runtime.ProcessInstance;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by fjm on 2018/1/3.
 */
@Transactional
@Service
public class ActivitiCancelServiceImpl implements IActiviCancelService{

    public static final String WORKFLOW_TYPE = "workflowType";
    @Autowired
    private HistoryService historyService;

    @Autowired
    private RuntimeService runtimeService;

    @Autowired
    private Map<String,IActivitiCommonService> serviceMap;

    @Override
    public void cancelWorkFlow(String procId, IRequest request) throws WorkflowTypeNullException {
        List<HistoricActivityInstance> historicActivityInstances = historyService.createHistoricActivityInstanceQuery()
                .processInstanceId(procId).orderByHistoricActivityInstanceStartTime().asc().list();
        HistoricActivityInstance lastHistoricActivityInstance = historicActivityInstances
                .get(historicActivityInstances.size() - 1);
        String executionId = lastHistoricActivityInstance.getExecutionId();
        Map<String, VariableInstance> variableInstances = runtimeService.getVariableInstances(executionId);
        final Map<String,IActivitiCommonService> actServiceMap = new HashMap<>();
        serviceMap.forEach((k,v)->{
            actServiceMap.put(v.getWorkFlowType(),v);
        });
        if (!variableInstances.containsKey(WORKFLOW_TYPE)){
            //修改为自定义异常
            throw new WorkflowTypeNullException();
        }else {
            VariableInstance variableInstance = variableInstances.get(WORKFLOW_TYPE);
            String textValue = variableInstance.getTextValue();
            IActivitiCommonService iActivitiCommonService = actServiceMap.get(textValue);
            ProcessInstance processInstance = runtimeService.createProcessInstanceQuery().processInstanceId(procId).singleResult();
            Map<String,Object> params = new HashMap<>();
            params.put("businessKey",processInstance.getBusinessKey());
            params.put("processInstanceId",procId);
            iActivitiCommonService.cancel(request,params);
        }
    }
}
