package com.hand.hap.activiti.service.impl;

import com.hand.hap.activiti.custom.IActivitiBean;
import hls.core.fnd.dto.HLSPosition;
import hls.core.fnd.dto.HlsEmployee;
import hls.core.fnd.mapper.HlsEmployeeMapper;
import hls.core.fnd.mapper.HlsPositionMapper;
import hls.core.prj.dto.HlsCusPrjProjectMeetingJudge;
import hls.core.prj.mapper.HlsCusPrjProjectMeetingJudgeMapper;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.impl.persistence.entity.UserEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.hand.hap.activiti.service.IActivitiEntityService;
import com.hand.hap.activiti.util.ActivitiUtils;
import java.util.ArrayList;
import java.util.List;

@Service
public class ActivitiEntityServiceImpl implements IActivitiEntityService, IActivitiBean {

    @Autowired
    private HlsEmployeeMapper employeeMapper;

    @Autowired
    private HlsPositionMapper positionMapper;

    @Autowired
    private HlsCusPrjProjectMeetingJudgeMapper projectMeetingJudgeMapper;

    @Override
    public String getDirector(String employeeCode) {
        List<HlsEmployee> director = employeeMapper.getDirector(employeeCode);
        if (director.size() == 0) {
            return "ADMIN";
        }
        return director.get(0).getEmployeeCode();
    }

    @Override
    public String getDeptDirector(String employeeCode){
        List<HlsEmployee> director = employeeMapper.getDeptDirector(employeeCode);
        if (director.size() == 0) {
            return "ADMIN";
        }
        return director.get(0).getEmployeeCode();
    }

    @Override
    public boolean isMemberOfGroup(String userId, String groupId) {
        return true;
    }

    @Override
    public String getName(String  userId) {
        return getEmployee(userId).getFirstName();
    }

    @Override
    public String getEmail(String userId) {
        return getEmployee(userId).getEmail();
    }

    @Override
    public UserEntity getEmployee(String employeeCode) {
        return ActivitiUtils.toActivitiUser(employeeMapper.queryByCode(employeeCode));
    }

    @Override
    public String getGroupName(String groupId) {
        HLSPosition pos = positionMapper.getPositionByCode(groupId);
        if (pos == null) {
            return groupId;
        }
        return pos.getPositionName();
    }

    @Override
    public String getEmployeeCode(String employeeCode) {
        return employeeCode;
    }

    @Override
    public List<String> getPositionEmp(String positionCode) {
        HLSPosition position = positionMapper.getPositionByCode(positionCode);
        List<String> emps = new ArrayList<String>();
        List<HlsEmployee> employees = employeeMapper.queryAllByPosition(position.getPositionId());
        for(HlsEmployee e:employees){
            emps.add(e.getEmployeeCode());
        }
        return emps;
    }

    @Override
    public List<String> getProjectMeetingEmp(DelegateExecution delegateExecution) {
        List<String> emps = new ArrayList<String>();
        HlsCusPrjProjectMeetingJudge projectMeetingJudge = new HlsCusPrjProjectMeetingJudge();
        projectMeetingJudge.setProjectId(Long.parseLong(delegateExecution.getParent().getProcessInstanceBusinessKey()));
        List<HlsCusPrjProjectMeetingJudge> projectMeetingJudges = projectMeetingJudgeMapper.select(projectMeetingJudge);

        for(int i=0;i<projectMeetingJudges.size();i++){
            HlsEmployee e = employeeMapper.selectEmployeeCode(projectMeetingJudges.get(i).getJudgeUserAllocationId());
            emps.add(e.getEmployeeCode());
        }
        return emps;
    }


    @Override
    public String getBeanName() {
        return "hlsEmpService";
    }
}
