package com.hand.hap.core.json;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.hand.hap.core.util.TimeZoneUtil;
import oracle.sql.TIMESTAMP;
import org.apache.commons.lang.time.DateFormatUtils;

import java.io.IOException;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.Date;

/**
 * 支持oracle.sql.TIMESTAMP序列化
 * Created by fjm on 2017/9/16.
 */
public class TimeStampSerializer extends JsonSerializer<TIMESTAMP> {

    private String pattern = "yyyy-MM-dd HH:mm:ss";

    /**
     * @return the pattern
     */
    public String getPattern() {
        return pattern;
    }

    /**
     * @param pattern
     *            the pattern to set
     */
    public void setPattern(String pattern) {
        this.pattern = pattern;
    }

    @Override
    public void serialize(TIMESTAMP timestamp, JsonGenerator jsonGenerator, SerializerProvider serializerProvider) throws IOException, JsonProcessingException {

        try {
            Timestamp ts = timestamp.timestampValue();
            Date date = new Date(ts.getTime());
            jsonGenerator.writeString(DateFormatUtils.format(date , getPattern(), TimeZoneUtil.getTimeZone()));
        } catch (SQLException e) {
            e.printStackTrace();
        }

    }

}
