package com.hand.hap.excel.utils;

import com.hand.hap.excel.dto.ColumnInfo;
import com.hand.hap.excel.dto.ExportConfig;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.xssf.streaming.SXSSFCell;
import org.apache.poi.xssf.streaming.SXSSFRow;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.ReflectionUtils;

import java.util.Date;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

public class ExcelUtils {
    private static Logger logger = LoggerFactory.getLogger(ExcelUtils.class);

    public static SXSSFSheet createSheet(SXSSFWorkbook wb, SXSSFSheet sheet, Object object, AtomicInteger count, AtomicInteger rowIndex, int rowMaxNumber, ExportConfig gridInfo, CellStyle dateFormat) {
        if (count.get() % rowMaxNumber == 0) {
            sheet = wb.createSheet();
            createHeaderRow(gridInfo.getColumnsInfo(), wb, sheet);
            rowIndex.set(0);
        }
        count.getAndIncrement();
        SXSSFRow row = sheet.createRow(rowIndex.getAndIncrement());
        createRow(gridInfo.getColumnsInfo(), object, row, dateFormat);
        return sheet;
    }


    public static void createRow(List<ColumnInfo> columnInfos, Object object, SXSSFRow row, CellStyle dateFormat) {
        for (int ii = 0; ii < columnInfos.size(); ii++) {
            Object fieldObject = null;
            try {
                fieldObject = PropertyUtils.getProperty(object, columnInfos.get(ii).getName());
            } catch (Exception e) {
                logger.error(e.getMessage(), e);
            }
            String type = columnInfos.get(ii).getType();

            SXSSFCell cell = row.createCell(ii);

            if (null == fieldObject) {
                cell.setCellType(XSSFCell.CELL_TYPE_STRING);
                cell.setCellValue((String) null);
            } else {
                switch (type.toUpperCase()) {
                    case "NUMBER":
                    case "FLOAT":
                        cell.setCellType(XSSFCell.CELL_TYPE_NUMERIC);
                        cell.setCellValue((Float) fieldObject);
                        break;
                    case "DOUBLE":
                        cell.setCellType(XSSFCell.CELL_TYPE_NUMERIC);
                        cell.setCellValue((Double) fieldObject);
                        break;
                    case "INT":
                    case "INTEGER":
                    case "LONG":
                        cell.setCellType(XSSFCell.CELL_TYPE_NUMERIC);
                        cell.setCellValue((Long) fieldObject);
                        break;
                    case "DATE":
                        cell.setCellStyle(dateFormat);
                        cell.setCellValue((Date) fieldObject);
                        break;
                    case "BOOLEAN":
                        cell.setCellType(XSSFCell.CELL_TYPE_BOOLEAN);
                        cell.setCellValue((Boolean) fieldObject);
                        break;
                    default:
                        cell.setCellType(XSSFCell.CELL_TYPE_STRING);
                        cell.setCellValue((String) fieldObject);
                        break;
                }
            }
        }
    }

    public static void createHeaderRow(List<ColumnInfo> columnInfos, SXSSFWorkbook wb, SXSSFSheet sheet) {
        SXSSFRow firstRow = sheet.createRow(0);
        // 设置列字体align
        CellStyle cellStyle = wb.createCellStyle();
        cellStyle.setAlignment(CellStyle.ALIGN_CENTER);
        for (int i = 0; i < columnInfos.size(); i++) {
            SXSSFCell firstCell = firstRow.createCell(i);
            firstCell.setCellValue(columnInfos.get(i).getTitle());
            // 设置列宽度
            sheet.setColumnWidth(i, columnInfos.get(i).getWidth() * 80);

            firstCell.setCellStyle(cellStyle);
        }
    }

    public static void initColumnType(List<ColumnInfo> columnInfos, Object object) {
        for (ColumnInfo columnInfo : columnInfos) {
            columnInfo.setType(ReflectionUtils.findField(object.getClass(), columnInfo.getName()).getType().getSimpleName());
        }

    }
}
