package com.hand.hap.system.controllers;

import com.hand.hap.account.dto.User;
import com.hand.hap.account.service.IUserService;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.ResponseData;
import com.hand.hap.system.dto.UserLogin;
import com.hand.hap.system.service.IUserLoginService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@Controller
public class UserLoginController extends BaseController{
    @Autowired
    private IUserLoginService service;

    @Autowired
    private IUserService userService;

    @RequestMapping(value = "/sys/user/login/query")
    @ResponseBody
    public ResponseData query(final UserLogin userLogin, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
        IRequest requestContext = createRequestContext(request);

        return new ResponseData(service.query(requestContext, userLogin, page, pagesize));
//        User u = new User();
//        UserLogin userLogin1 = new UserLogin();
//        if(userLogin.getUserName() !=null) {
//            u = userService.selectByUserName(userLogin.getUserName());
//            userLogin1.setUserId(u.getUserId());
//        }
//        List<UserLogin> list = service.select(requestContext,userLogin1,page,pagesize);
//
//        return new ResponseData(list);
    }

    @RequestMapping(value = "/sys/user/login/name/query")
    @ResponseBody
    public ResponseData queryByName(final User user, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
        IRequest requestContext = createRequestContext(request);
        User u = userService.selectByUserName(user.getUserName());
        UserLogin userLogin = new UserLogin();
        userLogin.setUserId(u.getUserId());
        List<UserLogin> list = service.select(requestContext,userLogin,page,pagesize);
        return new ResponseData(list);
    }

    @RequestMapping(value = "/sys/user/login/submit")
    @ResponseBody
    public ResponseData submit(@RequestBody final List<UserLogin> userLogins, final BindingResult result,
                               final HttpServletRequest request){

        getValidator().validate(userLogins, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.batchUpdate(requestContext, userLogins));
    }

    @RequestMapping(value="/sys/user/login/remove")
    @ResponseBody
    public ResponseData remove(@RequestBody final List<UserLogin> userLogins, final BindingResult result,
                               final HttpServletRequest request){
        service.batchDelete(userLogins);
        return new  ResponseData();
    }


}
