//
// Source code recreated from a .class file by IntelliJ IDEA
// (powered by Fernflower decompiler)
//

package com.hand.hap.account.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.account.dto.Role;
import com.hand.hap.account.dto.RoleExt;
import com.hand.hap.account.dto.User;
import com.hand.hap.account.exception.RoleException;
import com.hand.hap.account.mapper.RoleMapper;
import com.hand.hap.account.mapper.UserRoleMapper;
import com.hand.hap.account.service.IRole;
import com.hand.hap.account.service.IRoleService;
import com.hand.hap.cache.Cache;
import com.hand.hap.cache.CacheManager;
import com.hand.hap.core.IRequest;
import com.hand.hap.core.annotation.StdWho;
import com.hand.hap.function.service.IRoleFunctionService;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import java.util.Iterator;
import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

@Service
public class RoleServiceImpl extends BaseServiceImpl<Role> implements IRoleService {
    @Autowired
    private RoleMapper roleMapper;
    @Autowired
    private UserRoleMapper userRoleMapper;
    @Autowired
    private IRoleFunctionService roleFunctionService;
    @Autowired
    private CacheManager cacheManager;

    public RoleServiceImpl() {
    }



    @Transactional(
            propagation = Propagation.SUPPORTS
    )
    public List<IRole> queryRolesByUser(IRequest requestContext, User user) {
        List rootRoles = this.roleMapper.selectUserRoles(user.getUserId());
        return rootRoles;
    }


    @Transactional(
            propagation = Propagation.SUPPORTS
    )
    public List<IRole> selectRoleNotUserRoles(IRequest request, RoleExt roleExt, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);
        List list = this.roleMapper.selectRoleNotUserRoles(roleExt);
        return list;
    }

    @Transactional(
            propagation = Propagation.SUPPORTS
    )
    public List<IRole> selectRolesByUser(IRequest requestContext, User user) {
        List rootRoles = this.roleMapper.selectUserRoles(user.getUserId());
        return rootRoles;
    }

    @Transactional(
            propagation = Propagation.SUPPORTS
    )
    public List<IRole> selectUserRolesByUserId(Long userId) {
        List rootRoles = this.roleMapper.selectUserRolesByUserId(userId);
        return rootRoles;
    }

    @Transactional(
            propagation = Propagation.SUPPORTS
    )
    public void checkUserRoleExists(Long userId, Long roleId) throws RoleException {
        if(this.roleMapper.selectUserRoleCount(userId, roleId) != 1) {
            throw new RoleException("error.account_role_invalid", "error.account_role_invalid", (Object[])null);
        }
    }

    public IRole selectRoleByCode(String roleCode) {
        IRole role = (IRole)this.getRoleCache().getValue(roleCode);
        if(role == null) {
            Role record = new Role();
            record.setRoleCode(roleCode);
            role = (IRole)this.roleMapper.selectOne(record);
        }

        return role;
    }

    protected boolean useSelectiveUpdate() {
        return false;
    }

    public int deleteByPrimaryKey(Role record) {
        int ret = super.deleteByPrimaryKey(record);
        this.userRoleMapper.deleteByRoleId(record.getRoleId());
        this.roleFunctionService.clearRoleFunctionByRoleId(record.getRoleId());
        this.getRoleCache().remove(record.getRoleCode());
        return ret;
    }

    public Role updateByPrimaryKey(IRequest request, @StdWho Role record) {
        int count = this.roleMapper.updateByPrimaryKey(record);
        this.checkOvn(count, record);
        Role oldRecord = (Role)this.roleMapper.selectByPrimaryKey(record);
        if(!StringUtils.equals(oldRecord.getRoleCode(), record.getRoleCode())) {
            this.getRoleCache().remove(oldRecord.getRoleCode());
        }

        this.getRoleCache().setValue(record.getRoleCode(), record);
        return record;
    }

    public Role insertSelective(IRequest request, Role record) {
        int count = this.roleMapper.insertSelective(record);
        this.checkOvn(count, record);
        this.getRoleCache().setValue(record.getRoleCode(), record);
        return record;
    }

    public List<Role> batchUpdate(IRequest request, List<Role> list) {
        Iterator var3 = list.iterator();

        while(var3.hasNext()) {
            Role role = (Role)var3.next();
            if(role.getRoleId() == null) {
                ((IRoleService)this.self()).insertSelective(request, role);
            } else {
                ((IRoleService)this.self()).updateByPrimaryKey(request, role);
            }
        }

        return list;
    }

    Cache<IRole> getRoleCache() {
        return this.cacheManager.getCache("role");
    }
}
