//
// Source code recreated from a .class file by IntelliJ IDEA
// (powered by Fernflower decompiler)
//

package com.hand.hap.account.service.impl;

import com.hand.hap.account.constants.UserConstants;
import com.hand.hap.account.dto.User;
import com.hand.hap.account.dto.UserRole;
import com.hand.hap.account.exception.UserException;
import com.hand.hap.account.mapper.UserMapper;
import com.hand.hap.account.mapper.UserRoleMapper;
import com.hand.hap.account.service.IRole;
import com.hand.hap.account.service.IRoleService;
import com.hand.hap.account.service.IUserService;
import com.hand.hap.cache.impl.ResourceItemAssignCache;
import com.hand.hap.cache.impl.RoleFunctionCache;
import com.hand.hap.core.IRequest;
import com.hand.hap.core.annotation.StdWho;
import com.hand.hap.function.dto.Function;
import com.hand.hap.function.dto.MenuItem;
import com.hand.hap.function.dto.ResourceItemAssign;
import com.hand.hap.function.mapper.ResourceItemAssignMapper;
import com.hand.hap.function.service.IFunctionService;
import com.hand.hap.function.service.IRoleFunctionService;
import com.hand.hap.function.service.IRoleResourceItemService;
import com.hand.hap.mybatis.common.Criteria;
import com.hand.hap.security.IUserSecurityStrategy;
import com.hand.hap.security.PasswordManager;
import com.hand.hap.security.service.impl.UserSecurityStrategyManager;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.BiConsumer;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

@Service
@Transactional
public class UserServiceImpl extends BaseServiceImpl<User> implements IUserService {
    @Autowired
    private PasswordManager passwordManager;
    @Autowired
    private UserMapper userMapper;
    @Autowired
    UserSecurityStrategyManager userSecurityStrategyManager;
    @Autowired
    @Qualifier("roleServiceImpl")
    private IRoleService roleService;
    @Autowired
    private UserRoleMapper userRoleMapper;
    @Autowired
    private IFunctionService functionService;
    @Autowired
    private IRoleFunctionService roleFunctionService;
    @Autowired
    private IRoleResourceItemService roleResourceItemService;
    @Autowired
    private ResourceItemAssignCache resourceItemAssignCache;
    @Autowired
    private ResourceItemAssignMapper resourceItemAssignMapper;
    @Autowired
    private RoleFunctionCache roleFunctionCache;

    public UserServiceImpl() {
    }

    public User insertSelective(IRequest request, User record) {
        if(StringUtils.isEmpty(record.getPassword())) {
            record.setPassword(this.passwordManager.getDefaultPassword());
        }

        record.setPasswordEncrypted(this.passwordManager.encode(record.getPassword()));
        List userSecurityStrategies = this.userSecurityStrategyManager.getUserSecurityStrategyList();

        IUserSecurityStrategy userSecurityStrategy;
        for(Iterator var4 = userSecurityStrategies.iterator(); var4.hasNext(); record = userSecurityStrategy.beforeCreateUser(request, record)) {
            userSecurityStrategy = (IUserSecurityStrategy)var4.next();
        }

        record = (User)super.insertSelective(request, record);
        this.userMapper.updatePassword(record.getUserId(), record.getPasswordEncrypted());
        return record;
    }

    public void validateEmail(String email) throws UserException {
        if(!UserConstants.PATTERN_EMAIL_REGEX.matcher(email).matches()) {
            throw new UserException("error.user.email", new Object[0]);
        }
    }

    public void validatePhone(String phone) throws UserException {
        if(!UserConstants.PATTERN_PHONE_REGEX.matcher(phone).matches()) {
            throw new UserException("error.user.phone", new Object[0]);
        }
    }

    public User login(User user) throws UserException {
        if(user != null && !org.apache.commons.lang3.StringUtils.isAnyBlank(new CharSequence[]{user.getUserName(), user.getPassword()})) {
            User user1 = this.userMapper.selectByUserName(user.getUserName());
            if(user1 == null) {
                throw new UserException("error.login.name_password_not_match", "error.login.name_password_not_match", (Object[])null);
            } else if("LOCK".equals(user1.getStatus())) {
                throw new UserException("error.user.account_expired", "error.user.account_expired", (Object[])null);
            } else if(user1.getStartActiveDate() != null && user1.getStartActiveDate().getTime() > System.currentTimeMillis()) {
                throw new UserException("error.user.account_expired", "error.user.account_expired", (Object[])null);
            } else if(user1.getEndActiveDate() != null && user1.getEndActiveDate().getTime() < System.currentTimeMillis()) {
                throw new UserException("error.user.account_expired", "error.user.account_expired", (Object[])null);
            } else if(!this.passwordManager.matches(user.getPassword(), user1.getPasswordEncrypted())) {
                throw new UserException("error.login.name_password_not_match", "error.login.name_password_not_match", (Object[])null);
            } else {
                return user1;
            }
        } else {
            throw new UserException("error.login.name_password_not_match", "error.login.name_password_not_match", (Object[])null);
        }
    }

    public User selectByUserName(String userName) {
        return this.userMapper.selectByUserName(userName);
    }

    public List<User> selectUserNameByEmployeeCode(String employeeCode) {
        return this.userMapper.selectUserNameByEmployeeCode(employeeCode);
    }

    public User updateByPrimaryKeySelective(IRequest request, @StdWho User record) {
        int count = this.userMapper.updateBasic(record);
        this.checkOvn(count, record);
        return record;
    }

    @Transactional(
            propagation = Propagation.REQUIRED
    )
    public User update(IRequest request, User user) throws UserException {
        Criteria criteria = new Criteria(user);
        criteria.update(new String[]{"email", "email"});
        user = (User)((IUserService)this.self()).updateByPrimaryKeyOptions(request, user, criteria);
        return user;
    }

    @Transactional
    public void updatePassword(Long userId, String password) {
        String passwordEncrypted = this.passwordManager.encode(password);
        this.userMapper.updatePassword(userId, passwordEncrypted);
    }

    @Transactional(
            propagation = Propagation.SUPPORTS
    )
    public void validatePassword(IRequest request, String newPwd, String newPwdAgain) throws UserException {
        List userSecurityStrategies = this.userSecurityStrategyManager.getUserSecurityStrategyList();
        Iterator var5 = userSecurityStrategies.iterator();

        while(var5.hasNext()) {
            IUserSecurityStrategy userSecurityStrategy = (IUserSecurityStrategy)var5.next();
            userSecurityStrategy.validatePassword(newPwd, newPwdAgain);
        }

    }

    public void updateNotFirstLogin(Long userId, String status) {
        this.userMapper.updateFirstLogin(userId, status);
    }

    @Transactional(
            propagation = Propagation.SUPPORTS
    )
    public List<User> selectUsers(IRequest request, User user, int pageNum, int pageSize) {
        Criteria criteria = new Criteria(user);
        criteria.where(new String[]{"userName"});
        criteria.select(new String[]{"userId", "userName", "employeeId", "employeeName", "employeeCode", "email", "phone", "status", "startActiveDate", "endActiveDate", "description"});
        return super.selectOptions(request, user, criteria, Integer.valueOf(pageNum), Integer.valueOf(pageSize));
    }

    public List<IRole> selectUserAndRoles(IRequest request, User user, int pageNum, int pageSize) {
        List roles = this.roleService.selectRolesByUser(request, user);
        return roles;
    }

    private List<MenuItem> updateMenu(List<MenuItem> menus, Set<Long> userFunctions) {
        if(!CollectionUtils.isEmpty(menus) && !CollectionUtils.isEmpty(userFunctions)) {
            synchronized(menus) {
                Iterator iterator = menus.iterator();

                while(iterator.hasNext()) {
                    MenuItem menu = (MenuItem)iterator.next();
                    boolean hasChild = false;
                    if(CollectionUtils.isNotEmpty(menu.getChildren())) {
                        List hasFunction = this.updateMenu(menu.getChildren(), userFunctions);
                        if(CollectionUtils.isNotEmpty(hasFunction)) {
                            hasChild = true;
                        }
                    }

                    boolean hasFunction1 = false;
                    Iterator var8 = userFunctions.iterator();

                    while(var8.hasNext()) {
                        Long userFunction = (Long)var8.next();
                        if(menu.getId().equals(userFunction)) {
                            hasFunction1 = true;
                            break;
                        }
                    }

                    if(!hasFunction1 && !hasChild) {
                        iterator.remove();
                    }
                }

                return menus;
            }
        } else {
            return new ArrayList();
        }
    }

    private Set<Long> getUserFunctions(List<UserRole> allRoleIds) {
        if(CollectionUtils.isEmpty(allRoleIds)) {
            return null;
        } else {
            HashSet userFunctions = new HashSet();
            Iterator var3 = allRoleIds.iterator();

            while(var3.hasNext()) {
                UserRole userRole = (UserRole)var3.next();
                Long[] ids = this.roleFunctionService.getRoleFunctionById(userRole.getRoleId());
                if(!ArrayUtils.isEmpty(ids)) {
                    userFunctions.addAll(Arrays.asList(ids));
                }
            }

            return userFunctions;
        }
    }

    private List<UserRole> getAllRoleIds(Long userId) {
        UserRole userRole = new UserRole();
        userRole.setUserId(userId);
        List userRoles = this.userRoleMapper.select(userRole);
        return userRoles;
    }

    public List<MenuItem> queryFunction(IRequest requestContext, Long userId) {
        List menus = this.functionService.selectAllMenus(requestContext);
        Set userFunctions = null;
        if(userId != null) {
            List allRoleIds = this.getAllRoleIds(userId);
            userFunctions = this.getUserFunctions(allRoleIds);
        }

        return this.updateMenu(menus, userFunctions);
    }

    public List<MenuItem> queryResourceItems(IRequest requestContext, Long userId, Long functionId) {
        Function function = new Function();
        function.setFunctionId(functionId);
        List resources = this.roleResourceItemService.queryHtmlResources(requestContext, function);
        List menus = this.updateResourceMenu(this.roleResourceItemService.createResources(resources));
        return this.updateMenuCheck(menus, this.getAllAssignElementIds(userId));
    }

    private List<MenuItem> updateMenuCheck(List<MenuItem> menus, List<Long> allAssignElementIds) {
        if(!CollectionUtils.isEmpty(menus) && !CollectionUtils.isEmpty(allAssignElementIds)) {
            Iterator var3 = menus.iterator();

            while(true) {
                while(var3.hasNext()) {
                    MenuItem menuItem = (MenuItem)var3.next();
                    if(CollectionUtils.isNotEmpty(menuItem.getChildren())) {
                        this.updateMenuCheck(menuItem.getChildren(), allAssignElementIds);
                    }

                    Iterator var5 = allAssignElementIds.iterator();

                    while(var5.hasNext()) {
                        Long assignElementId = (Long)var5.next();
                        if(menuItem.getId().equals(assignElementId)) {
                            menuItem.setIschecked(Boolean.FALSE);
                            break;
                        }
                    }
                }

                return menus;
            }
        } else {
            return menus;
        }
    }

    private List<Long> getAllAssignElementIds(Long userId) {
        ArrayList allAssignElementIds = new ArrayList();
        if(userId != null) {
            List userRoles = this.getAllRoleIds(userId);
            if(CollectionUtils.isEmpty(userRoles)) {
                return null;
            }

            List roleAssignElementIds = this.getAllRoleAssignElementIds(userRoles);
            ResourceItemAssign[] userAssigns = this.resourceItemAssignCache.getValue("user_" + userId);
            if(ArrayUtils.isEmpty(userAssigns)) {
                if(CollectionUtils.isNotEmpty(roleAssignElementIds)) {
                    allAssignElementIds.addAll(roleAssignElementIds);
                }

                return allAssignElementIds;
            }

            ResourceItemAssign[] var6 = userAssigns;
            int roleAssignElementId = userAssigns.length;

            for(int check = 0; check < roleAssignElementId; ++check) {
                ResourceItemAssign userAssign = var6[check];
                if("N".equalsIgnoreCase(userAssign.getEnable())) {
                    allAssignElementIds.add(userAssign.getElementId());
                }
            }

            if(CollectionUtils.isNotEmpty(roleAssignElementIds)) {
                Iterator var13 = roleAssignElementIds.iterator();

                while(var13.hasNext()) {
                    Long var14 = (Long)var13.next();
                    boolean var15 = true;
                    ResourceItemAssign[] var16 = userAssigns;
                    int var10 = userAssigns.length;

                    for(int var11 = 0; var11 < var10; ++var11) {
                        ResourceItemAssign userAssign1 = var16[var11];
                        if(var14.equals(userAssign1.getElementId())) {
                            var15 = false;
                            break;
                        }
                    }

                    if(var15) {
                        allAssignElementIds.add(var14);
                    }
                }
            }
        }

        return allAssignElementIds;
    }

    private List<Long> getSameFunctions(Set<Long> allRoleFunctions, Set<Long> roleFunctions) {
        ArrayList sameFunctions = new ArrayList();
        Iterator var4 = allRoleFunctions.iterator();

        while(var4.hasNext()) {
            Long allRoleFunction = (Long)var4.next();
            Iterator var6 = roleFunctions.iterator();

            while(var6.hasNext()) {
                Long roleFunction = (Long)var6.next();
                if(allRoleFunction.equals(roleFunction)) {
                    sameFunctions.add(allRoleFunction);
                }
            }
        }

        return sameFunctions;
    }

    private List<Long> compareFunctionElements(List<Long> allFunctionElements, List<Long> functionElements) {
        if(!CollectionUtils.isEmpty(allFunctionElements) && !CollectionUtils.isEmpty(functionElements)) {
            allFunctionElements.retainAll(functionElements);
            return allFunctionElements;
        } else {
            return new ArrayList();
        }
    }

    private Map<Long, List<Long>> getFunctionElements(ResourceItemAssign[] assigns) {
        HashMap function = new HashMap();
        if(!ArrayUtils.isEmpty(assigns)) {
            ResourceItemAssign[] var3 = assigns;
            int var4 = assigns.length;

            for(int var5 = 0; var5 < var4; ++var5) {
                ResourceItemAssign assign = var3[var5];
                Object functionElements = (List)function.get(assign.getFunctionId());
                if(CollectionUtils.isEmpty((Collection)functionElements)) {
                    functionElements = new ArrayList();
                    function.put(assign.getFunctionId(), functionElements);
                }

                ((List)functionElements).add(assign.getElementId());
            }
        }

        return function;
    }

    private List<Long> getAllRoleAssignElementIds(List<UserRole> userRoles) {
        ArrayList allRoleAssignElementIds = new ArrayList();
        Object allFunctionElements = new HashMap();
        boolean firstRole = true;
        HashSet allRoleFunctions = new HashSet();
        Iterator var6 = userRoles.iterator();

        while(true) {
            while(true) {
                Long roleId;
                Long[] roleFunctionList;
                do {
                    if(!var6.hasNext()) {
                        ((Map)allFunctionElements).forEach((k, v) -> {
                            if(!CollectionUtils.isEmpty((Collection)v)) {
                                Iterator var3 = ((Collection)v).iterator();
                                while(var3.hasNext()) {
                                    Long assignElementId = (Long)var3.next();
                                    allRoleAssignElementIds.add(assignElementId);
                                }
                            }

                        });
                        return allRoleAssignElementIds;
                    }

                    UserRole userRole = (UserRole)var6.next();
                    roleId = userRole.getRoleId();
                    roleFunctionList = this.roleFunctionCache.getValue(roleId.toString());
                } while(ArrayUtils.isEmpty(roleFunctionList));

                HashSet roleFunctions = new HashSet();
                Long[] functionElements = roleFunctionList;
                int sameFunctions = roleFunctionList.length;

                Long sameFunction;
                for(int var13 = 0; var13 < sameFunctions; ++var13) {
                    sameFunction = functionElements[var13];
                    roleFunctions.add(sameFunction);
                }

                Map var18 = this.getFunctionElements(this.resourceItemAssignCache.getValue("role_" + roleId));
                if(firstRole) {
                    firstRole = false;
                    allRoleFunctions = roleFunctions;
                    allFunctionElements = var18;
                } else {
                    List var19 = this.getSameFunctions(allRoleFunctions, roleFunctions);
                    Iterator var20 = roleFunctions.iterator();

                    while(var20.hasNext()) {
                        sameFunction = (Long)var20.next();
                        boolean elements = false;
                        Iterator var16 = var19.iterator();

                        while(var16.hasNext()) {
                            Long sameFunction1 = (Long)var16.next();
                            if(sameFunction == sameFunction1) {
                                elements = true;
                                break;
                            }
                        }

                        if(!elements) {
                            ((Map)allFunctionElements).put(sameFunction, var18.get(sameFunction));
                        }
                    }

                    var20 = var19.iterator();

                    while(var20.hasNext()) {
                        sameFunction = (Long)var20.next();
                        List var21 = this.compareFunctionElements((List)((Map)allFunctionElements).get(sameFunction), (List)var18.get(sameFunction));
                        ((Map)allFunctionElements).put(sameFunction, var21);
                    }

                    allRoleFunctions.addAll(roleFunctions);
                }
            }
        }
    }

    private List<MenuItem> updateResourceMenu(List<MenuItem> menus) {
        if(CollectionUtils.isEmpty(menus)) {
            return menus;
        } else {
            this.removeVariableMenu(menus);
            synchronized(menus) {
                Iterator iterator = menus.iterator();

                while(iterator.hasNext()) {
                    MenuItem menu = (MenuItem)iterator.next();
                    if(CollectionUtils.isEmpty(menu.getChildren())) {
                        iterator.remove();
                    }
                }

                return menus;
            }
        }
    }

    private void removeVariableMenu(List<MenuItem> menus) {
        synchronized(menus) {
            Iterator iterator = menus.iterator();

            while(iterator.hasNext()) {
                MenuItem menu = (MenuItem)iterator.next();
                if("服务端变量".equalsIgnoreCase(menu.getText())) {
                    iterator.remove();
                    break;
                }

                if(CollectionUtils.isNotEmpty(menu.getChildren())) {
                    this.removeVariableMenu(menu.getChildren());
                }
            }

        }
    }

    public List<ResourceItemAssign> updateResourceItemAssign(IRequest requestContext, List<ResourceItemAssign> resourceItemAssignList, Long userId, Long functionId) {
        this.resourceItemAssignMapper.deleteByUserIdAndFunctionId(userId, functionId);
        if(CollectionUtils.isNotEmpty(resourceItemAssignList)) {
            Iterator var5 = resourceItemAssignList.iterator();

            while(var5.hasNext()) {
                ResourceItemAssign resourceItemAssign = (ResourceItemAssign)var5.next();
                resourceItemAssign.setAssignType("user");
                resourceItemAssign.setFunctionId(functionId);
                this.resourceItemAssignMapper.insertSelective(resourceItemAssign);
            }
        }

        this.resourceItemAssignCache.load("user_" + userId);
        return resourceItemAssignList;
    }

    public List<User> batchUpdate(IRequest request, List<User> list) {
        Criteria criteria = new Criteria();
        criteria.update(new String[]{"employeeId", "email", "phone", "status", "endActiveDate", "startActiveDate", "description"});
        Iterator var4 = list.iterator();

        while(var4.hasNext()) {
            User user = (User)var4.next();
            String var6 = user.get__status();
            byte var7 = -1;
            switch(var6.hashCode()) {
                case -838846263:
                    if(var6.equals("update")) {
                        var7 = 1;
                    }
                    break;
                case 96417:
                    if(var6.equals("add")) {
                        var7 = 0;
                    }
            }

            switch(var7) {
                case 0:
                    this.insertSelective(request, user);
                    break;
                case 1:
                    ((IUserService)this.self()).updateByPrimaryKeyOptions(request, user, criteria);
            }
        }

        return list;
    }

    public int deleteByPrimaryKey(User user) {
        int ret = super.deleteByPrimaryKey(user);
        this.userRoleMapper.deleteByUserId(user.getUserId());
        return ret;
    }

    @Transactional
    public void updateStatus(Long userId, String status) {
        this.userMapper.updateStatus(userId, status);
    }
}
