//
// Source code recreated from a .class file by IntelliJ IDEA
// (powered by Fernflower decompiler)
//

package com.hand.hap.attachment.service.impl;

import com.hand.hap.attachment.dto.AttachCategory;
import com.hand.hap.attachment.exception.CategoryNameRepeatException;
import com.hand.hap.attachment.exception.CategorySourceTypeRepeatException;
import com.hand.hap.attachment.exception.StoragePathNotExsitException;
import com.hand.hap.attachment.mapper.AttachCategoryMapper;
import com.hand.hap.attachment.service.IAttachCategoryService;
import com.hand.hap.core.IRequest;
import com.hand.hap.core.annotation.StdWho;
import com.hand.hap.core.util.FormatUtil;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Objects;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

@Service
@Transactional(
        rollbackFor = {Exception.class}
)
public class AttachCategoryServiceImpl implements IAttachCategoryService {
    public static final String CATEGORY_PATH_NOT_CORRECT = "category_path_not_correct";
    @Autowired
    private AttachCategoryMapper categoryMapper;

    public AttachCategoryServiceImpl() {
    }

    public AttachCategory insert(IRequest requestContext, @StdWho AttachCategory category) {
        if(StringUtils.isBlank(category.getStatus())) {
            category.setStatus("1");
        }

        AttachCategory parent = (AttachCategory)this.categoryMapper.selectByPrimaryKey(category.getParentCategoryId());
        if(parent != null && "1".equals(parent.getLeafFlag())) {
            parent.setLeafFlag("0");
            this.categoryMapper.updateByPrimaryKeySelective(parent);
        }

        this.categoryMapper.insertSelective(category);
        category.setPath(String.valueOf(category.getCategoryId()));
        if(category.getParentCategoryId() != null) {
            AttachCategory p1 = new AttachCategory();
            p1.setCategoryId(category.getParentCategoryId());
            p1 = (AttachCategory)this.categoryMapper.selectByPrimaryKey(p1);
            if(p1 != null) {
                category.setPath(p1.getPath() + "." + category.getCategoryId());
            }
        }

        this.categoryMapper.updateByPrimaryKeySelective(category);
        return category;
    }

    @Transactional(
            propagation = Propagation.SUPPORTS
    )
    public AttachCategory selectAttachByCode(IRequest requestContext, String sourceCode) {
        if(StringUtils.isBlank(sourceCode)) {
            return null;
        } else {
            AttachCategory params = new AttachCategory();
            params.setSourceType(sourceCode);
            params.setStatus("1");
            return this.categoryMapper.selectCategory(params);
        }
    }

    @Transactional(
            propagation = Propagation.SUPPORTS,
            rollbackFor = {Exception.class}
    )
    public String selectPathByCode(IRequest requestContext, String sourceType) throws StoragePathNotExsitException {
        AttachCategory params = new AttachCategory();
        params.setSourceType(sourceType);
        AttachCategory category = this.categoryMapper.selectCategory(params);
        if(category != null && category.getCategoryPath().length() > 0) {
            return category.getCategoryPath();
        } else {
            throw new StoragePathNotExsitException();
        }
    }

    @Transactional(
            propagation = Propagation.SUPPORTS
    )
    public List<AttachCategory> selectCategories(AttachCategory category) {
        return this.categoryMapper.select(category);
    }

    public List<AttachCategory> batchUpdate(IRequest requestCtx, @StdWho List<AttachCategory> categories) {
        Iterator var3 = categories.iterator();

        while(var3.hasNext()) {
            AttachCategory category = (AttachCategory)var3.next();
            if(category.getCategoryId() == null) {
                ((IAttachCategoryService)this.self()).insert(requestCtx, category);
            } else {
                this.categoryMapper.updateByPrimaryKeySelective(category);
            }
        }

        return categories;
    }

    @Transactional(
            propagation = Propagation.SUPPORTS
    )
    public AttachCategory selectByPrimaryKey(IRequest requestContext, Long categoryId) {
        return (AttachCategory)this.categoryMapper.selectByPrimaryKey(categoryId);
    }

    public boolean remove(AttachCategory category) {
        category = (AttachCategory)this.categoryMapper.selectByPrimaryKey(category.getCategoryId());
        if(category != null && !"0".equals(category.getLeafFlag())) {
            category.setStatus("2");
            this.categoryMapper.updateByPrimaryKeySelective(category);
            return true;
        } else {
            return false;
        }
    }

    @Transactional(
            propagation = Propagation.SUPPORTS
    )
    public List<AttachCategory> selectCategories(IRequest requestContext, AttachCategory category) {
        List<AttachCategory> categories = this.categoryMapper.select(category);
        categories.forEach((c) -> {
            String fileSize = FormatUtil.formatFileSize(c.getAllowedFileSize());
            if(!"0".equalsIgnoreCase(fileSize)) {
                c.setAllowedFileSizeDesc(fileSize);
            }

        });
        return categories;
    }

    public void validateType(List<AttachCategory> categories) throws CategorySourceTypeRepeatException,CategoryNameRepeatException  {
        Iterator var2 = categories.iterator();

        AttachCategory category;
        do {
            if(!var2.hasNext()) {
                return;
            }

            category = (AttachCategory)var2.next();
        } while(category.getCategoryId() != null || (this.categoryMapper.countSourceType(category.getSourceType()) <= 0 && this.categoryMapper.countCategoryName(category.getCategoryName(),category.getLeafFlag()) <= 0));

        if(this.categoryMapper.countSourceType(category.getSourceType()) >0){
            throw new CategorySourceTypeRepeatException(category.getSourceType());
        }
        if(this.categoryMapper.countCategoryName(category.getCategoryName(),category.getLeafFlag()) >0){
            throw new CategoryNameRepeatException(category.getCategoryName());
        }

    }

    public List<AttachCategory> selectCategoryBreadcrumbList(IRequest requestContext, Long categoryId) {
        AttachCategory a1 = new AttachCategory();
        a1.setCategoryId(categoryId);
        AttachCategory current = (AttachCategory)this.categoryMapper.selectByPrimaryKey(a1);
        String path = "";
        if(current != null) {
            path = StringUtils.trimToEmpty(current.getPath());
        }

        ArrayList idList = new ArrayList();
        idList.add(Long.valueOf(-1L));
        String[] paths = StringUtils.split(path, ".");

        for(int cates = 0; cates < paths.length; ++cates) {
            idList.add(Long.valueOf(Long.parseLong(paths[cates])));
        }

        List<AttachCategory> var9 = this.categoryMapper.selectAllParentCategory(idList);
        if(var9 != null && !var9.isEmpty()) {
            var9.sort((a, b) -> {
                return a.getCategoryId().equals(b.getParentCategoryId())?-1:(a.getParentCategoryId().equals(b.getCategoryId())?1:0);
            });
            var9.forEach((a) -> {
                a.setCategoryName(StringUtils.abbreviate(a.getCategoryName(), 15));
            });
        }

        return var9;
    }

    public List<AttachCategory> queryTree(IRequest requestContext, AttachCategory category) {
        List list = this.categoryMapper.queryTree(category);
        Iterator var4 = list.iterator();

        while(var4.hasNext()) {
            AttachCategory attachCategory = (AttachCategory)var4.next();
            if(Objects.equals(attachCategory.getParentCategoryId(), Long.valueOf(-1L))) {
                attachCategory.setParentCategoryId((Long)null);
            }
        }

        return list;
    }

    @Override
    public List<AttachCategory> queryAttach(IRequest iRequest, AttachCategory attachCategory) {
        return categoryMapper.queryAttach(attachCategory);
    }
}
