//
// Source code recreated from a .class file by IntelliJ IDEA
// (powered by Fernflower decompiler)
//

package com.hand.hap.security.captcha.impl;

import com.hand.hap.security.captcha.ICaptchaManager;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Random;
import java.util.UUID;
import java.util.concurrent.TimeUnit;
import javax.imageio.ImageIO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisCallback;
import org.springframework.data.redis.core.RedisTemplate;

public class DefaultCaptchaManager implements ICaptchaManager {
    private static final int CAPTCHA_WIDTH = 90;
    private static final int CAPTCHA_HEIGHT = 32;
    private static final int CAPTCHA_CODE_COUNT = 4;
    private static final int CAPTCHA_CODE_X = 19;
    private static final int CAPTCHA_CODE_Y = 26;
    private static final int CAPTCHA_FONT_HEIGHT = 28;
    private static final int CAPTCHA_EXPIRE = 300;
    private static final int LINE_COUNT = 20;
    private static final int LINE_DY = 12;
    private static final int MAX_RGB = 255;
    private static final char[] CAPTCHA_CODES = "ABCDEFGHJKLMNPQRSTUVWXYZ23456789".toCharArray();
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    private String category = "hap:cache:captcha";
    private String captchaKeyName = "captcha_key";
    private Integer expire = Integer.valueOf(300);

    public DefaultCaptchaManager() {
    }

    public String getCaptchaKeyName() {
        return this.captchaKeyName;
    }

    public void setCaptchaKeyName(String captchaKeyName) {
        this.captchaKeyName = captchaKeyName;
    }

    public Integer getExpire() {
        return this.expire;
    }

    public void setExpire(Integer expire) {
        this.expire = expire;
    }

    public String getCategory() {
        return this.category;
    }

    public void setCategory(String category) {
        this.category = category;
    }

    public String generateCaptchaCode() {
        StringBuilder randomCode = new StringBuilder();
        Random random = new Random();

        for(int i = 0; i < 4; ++i) {
            String code = String.valueOf(CAPTCHA_CODES[random.nextInt(CAPTCHA_CODES.length)]);
            randomCode.append(code);
        }

        return randomCode.toString();
    }

    public String generateCaptchaKey() {
        return UUID.randomUUID().toString();
    }

    public void generateCaptcha(String captchaKey, String captchaCode, OutputStream os) throws IOException {
        BufferedImage buffImg = new BufferedImage(90, 32, 1);
        Graphics gd = buffImg.getGraphics();
        Random random = new Random();
        gd.setColor(Color.WHITE);
        gd.fillRect(0, 0, 90, 32);
        Font font = new Font("Fixedsys", 1, 28);
        gd.setFont(font);
        gd.drawRect(0, 0, 89, 31);
        gd.setColor(Color.BLACK);

        int red;
        int green;
        int blue;
        int sw;
        int ignored;
        for(red = 0; red < 20; ++red) {
            green = random.nextInt(90);
            blue = random.nextInt(32);
            sw = random.nextInt(12);
            ignored = random.nextInt(12);
            gd.drawLine(green, blue, green + sw, blue + ignored);
        }

        boolean var24 = false;
        boolean var25 = false;
        boolean var26 = false;
        sw = Math.floorDiv(90, captchaCode.length());

        for(ignored = 0; ignored < captchaCode.length(); ++ignored) {
            String code = String.valueOf(captchaCode.charAt(ignored));
            red = random.nextInt(255);
            green = random.nextInt(255);
            blue = random.nextInt(255);
            gd.setColor(new Color(red, green, blue));
            gd.drawString(code, ignored * sw, 26);
        }

        this.redisTemplate.opsForValue().set(this.getCategory() + ":" + captchaKey, captchaCode, (long)this.getExpire().intValue(), TimeUnit.SECONDS);
        OutputStream var27 = os;
        Throwable var28 = null;

        try {
            ImageIO.write(buffImg, "jpeg", os);
        } catch (Throwable var22) {
            var28 = var22;
            throw var22;
        } finally {
            if(os != null) {
                if(var28 != null) {
                    try {
                        var27.close();
                    } catch (Throwable var21) {
                        var28.addSuppressed(var21);
                    }
                } else {
                    os.close();
                }
            }

        }

    }

    public boolean checkCaptcha(String captchaKey, String captchaCode) {
        if(captchaCode == null) {
            return false;
        } else {
            String key = this.getCategory() + ":" + captchaKey;
            String captchaCodeInRedis = (String)this.redisTemplate.opsForValue().get(key);
            this.removeCaptcha(key);
            return captchaCode.equalsIgnoreCase(captchaCodeInRedis);
        }
    }

    public void removeCaptcha(String key) {
        this.redisTemplate.execute((RedisCallback<Object>) (connection) -> {
            connection.del(new byte[][]{key.getBytes()});
            return null;
        });
    }
}
